@@ -78,11 +78,11 @@
  */
 static UINT cliprdr_server_packet_send(CliprdrServerPrivate* cliprdr, wStream* s)
 {
-	UINT rc;
-	size_t pos;
-	BOOL status;
-	UINT32 dataLen;
-	ULONG written;
+	UINT rc = 0;
+	size_t pos = 0;
+	BOOL status = 0;
+	UINT32 dataLen = 0;
+	ULONG written = 0;
 
 	WINPR_ASSERT(cliprdr);
 
@@ -119,14 +119,11 @@ static UINT cliprdr_server_capabilities(CliprdrServerContext* context,
                                         const CLIPRDR_CAPABILITIES* capabilities)
 {
 	size_t offset = 0;
-	UINT32 x;
-	wStream* s;
-	CliprdrServerPrivate* cliprdr;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(capabilities);
 
-	cliprdr = (CliprdrServerPrivate*)context->handle;
+	CliprdrServerPrivate* cliprdr = (CliprdrServerPrivate*)context->handle;
 
 	if (capabilities->common.msgType != CB_CLIP_CAPS)
 		WLog_WARN(TAG, "called with invalid type %08" PRIx32, capabilities->common.msgType);
@@ -137,7 +134,7 @@ static UINT cliprdr_server_capabilities(CliprdrServerContext* context,
 		return ERROR_INVALID_PARAMETER;
 	}
 
-	s = cliprdr_packet_new(CB_CLIP_CAPS, 0, 4 + CB_CAPSTYPE_GENERAL_LEN);
+	wStream* s = cliprdr_packet_new(CB_CLIP_CAPS, 0, 4 + CB_CAPSTYPE_GENERAL_LEN);
 
 	if (!s)
 	{
@@ -148,7 +145,7 @@ static UINT cliprdr_server_capabilities(CliprdrServerContext* context,
 	Stream_Write_UINT16(s,
 	                    (UINT16)capabilities->cCapabilitiesSets); /* cCapabilitiesSets (2 bytes) */
 	Stream_Write_UINT16(s, 0);                                    /* pad1 (2 bytes) */
-	for (x = 0; x < capabilities->cCapabilitiesSets; x++)
+	for (UINT32 x = 0; x < capabilities->cCapabilitiesSets; x++)
 	{
 		const CLIPRDR_CAPABILITY_SET* cap =
 		    (const CLIPRDR_CAPABILITY_SET*)(((const BYTE*)capabilities->capabilitySets) + offset);
@@ -175,6 +172,7 @@ static UINT cliprdr_server_capabilities(CliprdrServerContext* context,
 				if (!Stream_SafeSeek(s, cap->capabilitySetLength))
 				{
 					WLog_ERR(TAG, "short stream");
+					Stream_Free(s, TRUE);
 					return ERROR_NO_DATA;
 				}
 				break;
@@ -192,8 +190,8 @@ static UINT cliprdr_server_capabilities(CliprdrServerContext* context,
 static UINT cliprdr_server_monitor_ready(CliprdrServerContext* context,
                                          const CLIPRDR_MONITOR_READY* monitorReady)
 {
-	wStream* s;
-	CliprdrServerPrivate* cliprdr;
+	wStream* s = NULL;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(monitorReady);
@@ -224,8 +222,8 @@ static UINT cliprdr_server_monitor_ready(CliprdrServerContext* context,
 static UINT cliprdr_server_format_list(CliprdrServerContext* context,
                                        const CLIPRDR_FORMAT_LIST* formatList)
 {
-	wStream* s;
-	CliprdrServerPrivate* cliprdr;
+	wStream* s = NULL;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(formatList);
@@ -252,8 +250,8 @@ static UINT
 cliprdr_server_format_list_response(CliprdrServerContext* context,
                                     const CLIPRDR_FORMAT_LIST_RESPONSE* formatListResponse)
 {
-	wStream* s;
-	CliprdrServerPrivate* cliprdr;
+	wStream* s = NULL;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(formatListResponse);
@@ -283,8 +281,8 @@ cliprdr_server_format_list_response(CliprdrServerContext* context,
 static UINT cliprdr_server_lock_clipboard_data(CliprdrServerContext* context,
                                                const CLIPRDR_LOCK_CLIPBOARD_DATA* lockClipboardData)
 {
-	wStream* s;
-	CliprdrServerPrivate* cliprdr;
+	wStream* s = NULL;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(lockClipboardData);
@@ -314,8 +312,8 @@ static UINT
 cliprdr_server_unlock_clipboard_data(CliprdrServerContext* context,
                                      const CLIPRDR_UNLOCK_CLIPBOARD_DATA* unlockClipboardData)
 {
-	wStream* s;
-	CliprdrServerPrivate* cliprdr;
+	wStream* s = NULL;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(unlockClipboardData);
@@ -345,8 +343,8 @@ cliprdr_server_unlock_clipboard_data(CliprdrServerContext* context,
 static UINT cliprdr_server_format_data_request(CliprdrServerContext* context,
                                                const CLIPRDR_FORMAT_DATA_REQUEST* formatDataRequest)
 {
-	wStream* s;
-	CliprdrServerPrivate* cliprdr;
+	wStream* s = NULL;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(formatDataRequest);
@@ -378,8 +376,8 @@ static UINT
 cliprdr_server_format_data_response(CliprdrServerContext* context,
                                     const CLIPRDR_FORMAT_DATA_RESPONSE* formatDataResponse)
 {
-	wStream* s;
-	CliprdrServerPrivate* cliprdr;
+	wStream* s = NULL;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(formatDataResponse);
@@ -412,8 +410,8 @@ static UINT
 cliprdr_server_file_contents_request(CliprdrServerContext* context,
                                      const CLIPRDR_FILE_CONTENTS_REQUEST* fileContentsRequest)
 {
-	wStream* s;
-	CliprdrServerPrivate* cliprdr;
+	wStream* s = NULL;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(fileContentsRequest);
@@ -444,8 +442,8 @@ static UINT
 cliprdr_server_file_contents_response(CliprdrServerContext* context,
                                       const CLIPRDR_FILE_CONTENTS_RESPONSE* fileContentsResponse)
 {
-	wStream* s;
-	CliprdrServerPrivate* cliprdr;
+	wStream* s = NULL;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(fileContentsResponse);
@@ -514,12 +512,12 @@ static UINT cliprdr_server_receive_general_capability(CliprdrServerContext* cont
 static UINT cliprdr_server_receive_capabilities(CliprdrServerContext* context, wStream* s,
                                                 const CLIPRDR_HEADER* header)
 {
-	UINT16 capabilitySetType;
-	UINT16 capabilitySetLength;
+	UINT16 capabilitySetType = 0;
+	UINT16 capabilitySetLength = 0;
 	UINT error = ERROR_INVALID_DATA;
 	size_t cap_sets_size = 0;
 	CLIPRDR_CAPABILITIES capabilities = { 0 };
-	CLIPRDR_CAPABILITY_SET* capSet;
+	CLIPRDR_CAPABILITY_SET* capSet = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_UNUSED(header);
@@ -594,9 +592,9 @@ out:
 static UINT cliprdr_server_receive_temporary_directory(CliprdrServerContext* context, wStream* s,
                                                        const CLIPRDR_HEADER* header)
 {
-	size_t length;
+	size_t length = 0;
 	CLIPRDR_TEMP_DIRECTORY tempDirectory = { 0 };
-	CliprdrServerPrivate* cliprdr;
+	CliprdrServerPrivate* cliprdr = NULL;
 	UINT error = CHANNEL_RC_OK;
 
 	WINPR_ASSERT(context);
@@ -899,15 +897,16 @@ static UINT cliprdr_server_receive_filecontents_response(CliprdrServerContext* c
 static UINT cliprdr_server_receive_pdu(CliprdrServerContext* context, wStream* s,
                                        const CLIPRDR_HEADER* header)
 {
-	UINT error;
+	UINT error = 0;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(header);
 
-	WLog_DBG(TAG,
-	         "CliprdrServerReceivePdu: msgType: %" PRIu16 " msgFlags: 0x%04" PRIX16
-	         " dataLen: %" PRIu32 "",
-	         header->msgType, header->msgFlags, header->dataLen);
+	char buffer1[64] = { 0 };
+	char buffer2[64] = { 0 };
+	WLog_DBG(TAG, "CliprdrServerReceivePdu: msgType: %s, msgFlags: %s dataLen: %" PRIu32 "",
+	         CB_MSG_TYPE_STRING(header->msgType, buffer1, sizeof(buffer1)),
+	         CB_MSG_FLAGS_STRING(header->msgFlags, buffer2, sizeof(buffer2)), header->dataLen);
 
 	switch (header->msgType)
 	{
@@ -1068,14 +1067,14 @@ static UINT cliprdr_server_init(CliprdrServerContext* context)
  */
 static UINT cliprdr_server_read(CliprdrServerContext* context)
 {
-	wStream* s;
-	size_t position;
-	DWORD BytesToRead;
-	DWORD BytesReturned;
+	wStream* s = NULL;
+	size_t position = 0;
+	DWORD BytesToRead = 0;
+	DWORD BytesReturned = 0;
 	CLIPRDR_HEADER header = { 0 };
-	CliprdrServerPrivate* cliprdr;
-	UINT error;
-	DWORD status;
+	CliprdrServerPrivate* cliprdr = NULL;
+	UINT error = 0;
+	DWORD status = 0;
 
 	WINPR_ASSERT(context);
 
@@ -1216,9 +1215,9 @@ static DWORD WINAPI cliprdr_server_thread(LPVOID arg)
 	DWORD status = 0;
 	DWORD nCount = 0;
 	HANDLE events[MAXIMUM_WAIT_OBJECTS] = { 0 };
-	HANDLE ChannelEvent;
+	HANDLE ChannelEvent = NULL;
 	CliprdrServerContext* context = (CliprdrServerContext*)arg;
-	CliprdrServerPrivate* cliprdr;
+	CliprdrServerPrivate* cliprdr = NULL;
 	UINT error = CHANNEL_RC_OK;
 
 	WINPR_ASSERT(context);
@@ -1300,7 +1299,7 @@ static UINT cliprdr_server_open(CliprdrServerContext* context)
 {
 	void* buffer = NULL;
 	DWORD BytesReturned = 0;
-	CliprdrServerPrivate* cliprdr;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 
@@ -1347,7 +1346,7 @@ static UINT cliprdr_server_open(CliprdrServerContext* context)
  */
 static UINT cliprdr_server_close(CliprdrServerContext* context)
 {
-	CliprdrServerPrivate* cliprdr;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 
@@ -1370,8 +1369,8 @@ static UINT cliprdr_server_close(CliprdrServerContext* context)
  */
 static UINT cliprdr_server_start(CliprdrServerContext* context)
 {
-	UINT error;
-	CliprdrServerPrivate* cliprdr;
+	UINT error = 0;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 
@@ -1412,7 +1411,7 @@ static UINT cliprdr_server_start(CliprdrServerContext* context)
 static UINT cliprdr_server_stop(CliprdrServerContext* context)
 {
 	UINT error = CHANNEL_RC_OK;
-	CliprdrServerPrivate* cliprdr;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 
@@ -1442,7 +1441,7 @@ static UINT cliprdr_server_stop(CliprdrServerContext* context)
 
 static HANDLE cliprdr_server_get_event_handle(CliprdrServerContext* context)
 {
-	CliprdrServerPrivate* cliprdr;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	WINPR_ASSERT(context);
 
@@ -1463,7 +1462,7 @@ static UINT cliprdr_server_check_event_handle(CliprdrServerContext* context)
 
 CliprdrServerContext* cliprdr_server_context_new(HANDLE vcm)
 {
-	CliprdrServerPrivate* cliprdr;
+	CliprdrServerPrivate* cliprdr = NULL;
 	CliprdrServerContext* context = (CliprdrServerContext*)calloc(1, sizeof(CliprdrServerContext));
 
 	if (context)
@@ -1513,7 +1512,7 @@ CliprdrServerContext* cliprdr_server_context_new(HANDLE vcm)
 
 void cliprdr_server_context_free(CliprdrServerContext* context)
 {
-	CliprdrServerPrivate* cliprdr;
+	CliprdrServerPrivate* cliprdr = NULL;
 
 	if (!context)
 		return;
