@@ -396,14 +396,13 @@ static void dvcman_channel_free(DVCMAN_CHANNEL* channel)
 
 static void dvcman_channel_unref(DVCMAN_CHANNEL* channel)
 {
-	DVCMAN* dvcman = NULL;
-
 	WINPR_ASSERT(channel);
 	if (InterlockedDecrement(&channel->refCounter))
 		return;
 
-	dvcman = channel->dvcman;
-	HashTable_Remove(dvcman->channelsById, &channel->channel_id);
+	DVCMAN* dvcman = channel->dvcman;
+	if (dvcman)
+		HashTable_Remove(dvcman->channelsById, &channel->channel_id);
 }
 
 static UINT dvcchannel_send_close(DVCMAN_CHANNEL* channel)
@@ -457,7 +456,6 @@ static UINT dvcman_call_on_receive(DVCMAN_CHANNEL* channel, wStream* data)
 static UINT dvcman_channel_close(DVCMAN_CHANNEL* channel, BOOL perRequest, BOOL fromHashTableFn)
 {
 	UINT error = CHANNEL_RC_OK;
-	drdynvcPlugin* drdynvc = NULL;
 	DrdynvcClientContext* context = NULL;
 
 	WINPR_ASSERT(channel);
@@ -466,20 +464,24 @@ static UINT dvcman_channel_close(DVCMAN_CHANNEL* channel, BOOL perRequest, BOOL
 		case DVC_CHANNEL_INIT:
 			break;
 		case DVC_CHANNEL_RUNNING:
-			drdynvc = channel->dvcman->drdynvc;
-			context = drdynvc->context;
-			if (perRequest)
-				WLog_Print(drdynvc->log, WLOG_DEBUG, "sending close confirm for '%s'",
-				           channel->channel_name);
-
-			error = dvcchannel_send_close(channel);
-			if (error != CHANNEL_RC_OK)
+			if (channel->dvcman)
 			{
-				const char* msg = "error when sending close confirm for '%s'";
+				drdynvcPlugin* drdynvc = channel->dvcman->drdynvc;
+				WINPR_ASSERT(drdynvc);
+				context = drdynvc->context;
 				if (perRequest)
-					msg = "error when sending closeRequest for '%s'";
+					WLog_Print(drdynvc->log, WLOG_DEBUG, "sending close confirm for '%s'",
+					           channel->channel_name);
 
-				WLog_Print(drdynvc->log, WLOG_DEBUG, msg, channel->channel_name);
+				error = dvcchannel_send_close(channel);
+				if (error != CHANNEL_RC_OK)
+				{
+					const char* msg = "error when sending close confirm for '%s'";
+					if (perRequest)
+						msg = "error when sending closeRequest for '%s'";
+
+					WLog_Print(drdynvc->log, WLOG_DEBUG, msg, channel->channel_name);
+				}
 			}
 
 			channel->state = DVC_CHANNEL_CLOSED;
@@ -1723,6 +1725,9 @@ static UINT drdynvc_virtual_channel_event_connected(drdynvcPlugin* drdynvc, LPVO
 			WLog_Print(drdynvc->log, WLOG_ERROR, "CreateThread failed!");
 			goto error;
 		}
+
+		if (!SetThreadPriority(drdynvc->thread, THREAD_PRIORITY_HIGHEST))
+			WLog_Print(drdynvc->log, WLOG_WARN, "SetThreadPriority failed, ignoring.");
 	}
 
 error:
