@@ -874,6 +874,11 @@ static UINT rdpgfx_send_cache_offer(RDPGFX_PLUGIN* gfx)
 	}
 
 	count = persistent_cache_get_count(persistent);
+	if (count < 0)
+	{
+		error = ERROR_INVALID_DATA;
+		goto fail;
+	}
 
 	if (count >= RDPGFX_CACHE_ENTRY_MAX_COUNT)
 		count = RDPGFX_CACHE_ENTRY_MAX_COUNT - 1;
@@ -1179,6 +1184,7 @@ static UINT rdpgfx_recv_end_frame_pdu(GENERIC_CHANNEL_CALLBACK* callback, wStrea
 	Stream_Read_UINT32(s, pdu.frameId); /* frameId (4 bytes) */
 	DEBUG_RDPGFX(gfx->log, "RecvEndFramePdu: frameId: %" PRIu32 "", pdu.frameId);
 
+	const UINT64 start = GetTickCount64();
 	if (context)
 	{
 		IFCALLRET(context->EndFrame, error, context, &pdu);
@@ -1190,7 +1196,8 @@ static UINT rdpgfx_recv_end_frame_pdu(GENERIC_CHANNEL_CALLBACK* callback, wStrea
 			return error;
 		}
 	}
-
+	const UINT64 end = GetTickCount64();
+	const UINT64 EndFrameTime = end - start;
 	gfx->TotalDecodedFrames++;
 
 	if (!gfx->sendFrameAcks)
@@ -1230,7 +1237,7 @@ static UINT rdpgfx_recv_end_frame_pdu(GENERIC_CHANNEL_CALLBACK* callback, wStrea
 		case RDPGFX_CAPVERSION_107:
 			if (freerdp_settings_get_bool(gfx->rdpcontext->settings, FreeRDP_GfxSendQoeAck))
 			{
-				RDPGFX_QOE_FRAME_ACKNOWLEDGE_PDU qoe;
+				RDPGFX_QOE_FRAME_ACKNOWLEDGE_PDU qoe = { 0 };
 				UINT64 diff = (GetTickCount64() - gfx->StartDecodingTime);
 
 				if (diff > 65000)
@@ -1239,7 +1246,7 @@ static UINT rdpgfx_recv_end_frame_pdu(GENERIC_CHANNEL_CALLBACK* callback, wStrea
 				qoe.frameId = pdu.frameId;
 				qoe.timestamp = gfx->StartDecodingTime;
 				qoe.timeDiffSE = diff;
-				qoe.timeDiffEDR = 1;
+				qoe.timeDiffEDR = EndFrameTime;
 
 				if ((error = rdpgfx_send_qoe_frame_acknowledge_pdu(context, &qoe)))
 					WLog_Print(gfx->log, WLOG_ERROR,
@@ -2077,7 +2084,6 @@ static UINT rdpgfx_recv_pdu(GENERIC_CHANNEL_CALLBACK* callback, wStream* s)
  */
 static UINT rdpgfx_on_data_received(IWTSVirtualChannelCallback* pChannelCallback, wStream* data)
 {
-	wStream* s = NULL;
 	int status = 0;
 	UINT32 DstSize = 0;
 	BYTE* pDstData = NULL;
@@ -2093,14 +2099,17 @@ static UINT rdpgfx_on_data_received(IWTSVirtualChannelCallback* pChannelCallback
 	if (status < 0)
 	{
 		WLog_Print(gfx->log, WLOG_ERROR, "zgfx_decompress failure! status: %d", status);
+		free(pDstData);
 		return ERROR_INTERNAL_ERROR;
 	}
 
-	s = Stream_New(pDstData, DstSize);
+	wStream sbuffer = { 0 };
+	wStream* s = Stream_StaticConstInit(&sbuffer, pDstData, DstSize);
 
 	if (!s)
 	{
 		WLog_Print(gfx->log, WLOG_ERROR, "calloc failed!");
+		free(pDstData);
 		return CHANNEL_RC_NO_MEMORY;
 	}
 
@@ -2114,7 +2123,7 @@ static UINT rdpgfx_on_data_received(IWTSVirtualChannelCallback* pChannelCallback
 		}
 	}
 
-	Stream_Free(s, TRUE);
+	free(pDstData);
 	return error;
 }
 
@@ -2339,6 +2348,8 @@ static UINT init_plugin_cb(GENERIC_DYNVC_PLUGIN* base, rdpContext* rcontext, rdp
 		return CHANNEL_RC_NO_MEMORY;
 	}
 
+	gfx->suspendFrameAcks =
+	    freerdp_settings_get_bool(gfx->rdpcontext->settings, FreeRDP_GfxSuspendFrameAck);
 	gfx->MaxCacheSlots =
 	    freerdp_settings_get_bool(gfx->rdpcontext->settings, FreeRDP_GfxSmallCache) ? 4096 : 25600;
 
