@@ -70,6 +70,13 @@ typedef struct
 	char* responseMime;
 } wlf_request;
 
+typedef struct
+{
+	const FILE* responseFile;
+	UINT32 responseFormat;
+	const char* responseMime;
+} wlf_const_request;
+
 struct wlf_clipboard
 {
 	wlfContext* wfc;
@@ -218,7 +225,7 @@ static UINT wlf_cliprdr_send_client_format_list(wfClipboard* clipboard)
 {
 	WINPR_ASSERT(clipboard);
 
-	const CLIPRDR_FORMAT_LIST formatList = { .common.msgFlags = CB_RESPONSE_OK,
+	const CLIPRDR_FORMAT_LIST formatList = { .common.msgFlags = 0,
 		                                     .numFormats = (UINT32)clipboard->numClientFormats,
 		                                     .formats = clipboard->clientFormats,
 		                                     .common.msgType = CB_FORMAT_LIST };
@@ -311,7 +318,7 @@ static BOOL wlf_cliprdr_add_client_format(wfClipboard* clipboard, const char* mi
  *
  * @return 0 on success, otherwise a Win32 error code
  */
-static UINT wlf_cliprdr_send_data_request(wfClipboard* clipboard, const wlf_request* rq)
+static UINT wlf_cliprdr_send_data_request(wfClipboard* clipboard, const wlf_const_request* rq)
 {
 	WINPR_ASSERT(rq);
 
@@ -522,7 +529,7 @@ static void wlf_cliprdr_transfer_data(UwacSeat* seat, void* context, const char*
 
 	EnterCriticalSection(&clipboard->lock);
 
-	wlf_request request = { 0 };
+	wlf_const_request request = { 0 };
 	if (wlf_mime_is_html(mime))
 	{
 		request.responseMime = mime_html;
@@ -720,7 +727,7 @@ wlf_cliprdr_server_format_data_request(CliprdrClientContext* context,
                                        const CLIPRDR_FORMAT_DATA_REQUEST* formatDataRequest)
 {
 	UINT rc = CHANNEL_RC_OK;
-	BYTE* data = NULL;
+	char* data = NULL;
 	size_t size = 0;
 	const char* mime = NULL;
 	UINT32 formatId = 0;
@@ -849,6 +856,7 @@ wlf_cliprdr_server_format_data_response(CliprdrClientContext* context,
 	ClipboardLock(clipboard->system);
 	EnterCriticalSection(&clipboard->lock);
 
+	BYTE* cdata = NULL;
 	UINT32 srcFormatId = 0;
 	UINT32 dstFormatId = 0;
 	switch (request->responseFormat)
@@ -895,14 +903,14 @@ wlf_cliprdr_server_format_data_response(CliprdrClientContext* context,
 
 	const BOOL sres = ClipboardSetData(clipboard->system, srcFormatId, data, size);
 	if (sres)
-		data = ClipboardGetData(clipboard->system, dstFormatId, &len);
+		cdata = ClipboardGetData(clipboard->system, dstFormatId, &len);
 
-	if (!sres || !data)
+	if (!sres || !cdata)
 		goto unlock;
 
 	if (request->responseFile)
 	{
-		const size_t res = fwrite(data, 1, len, request->responseFile);
+		const size_t res = fwrite(cdata, 1, len, request->responseFile);
 		if (res == len)
 			rc = CHANNEL_RC_OK;
 	}
@@ -910,6 +918,7 @@ wlf_cliprdr_server_format_data_response(CliprdrClientContext* context,
 		rc = CHANNEL_RC_OK;
 
 unlock:
+	free(cdata);
 	ClipboardUnlock(clipboard->system);
 	LeaveCriticalSection(&clipboard->lock);
 fail:
