@@ -31,6 +31,25 @@
 #include "../xf_client.h"
 #include "../xfreerdp.h"
 
+static void xfreerdp_print_help(void)
+{
+	printf("Keyboard Shortcuts:\n");
+	printf("\t<Right CTRL>\n");
+	printf("\t\treleases keyboard and mouse grab\n");
+	printf("\t<CTRL>+<ALT>+<Return>\n");
+	printf("\t\ttoggles fullscreen state of the application\n");
+	printf("\t<CTRL>+<ALT>+c\n");
+	printf("\t\ttoggles remote control in a remote assistance session\n");
+	printf("\tAction Script\n");
+	printf("\t\tExecutes a predefined script on key press.\n");
+	printf("\t\tShould the script not exist it is ignored.\n");
+	printf("\t\tScripts can be provided at the default location ~/.config/freerdp/action.sh or as "
+	       "command line argument /action:script:<path>\n");
+	printf("\t\tThe script will receive the current key combination as argument.\n");
+	printf("\t\tThe output of the script is parsed for 'key-local' which tells that the script "
+	       "used the key combination, otherwise the combination is forwarded to the remote.\n");
+}
+
 int main(int argc, char* argv[])
 {
 	int rc = 1;
@@ -59,6 +78,8 @@ int main(int argc, char* argv[])
 	{
 		rc = freerdp_client_settings_command_line_status_print(settings, status, argc, argv);
 
+		xfreerdp_print_help();
+
 		if (freerdp_settings_get_bool(settings, FreeRDP_ListMonitors))
 			xf_list_monitors(xfc);
 
