@@ -124,6 +124,7 @@ struct xf_clipboard
 	BOOL incr_starts;
 	BYTE* incr_data;
 	int incr_data_length;
+	long event_mask;
 
 	/* XFixes extension */
 	int xfixes_event_base;
@@ -181,7 +182,7 @@ static xfCachedData* xf_cached_data_new(BYTE* data, UINT32 data_length)
 	return cached_data;
 }
 
-static xfCachedData* xf_cached_data_new_copy(BYTE* data, UINT32 data_length)
+static xfCachedData* xf_cached_data_new_copy(const BYTE* data, size_t data_length)
 {
 	BYTE* copy = NULL;
 	if (data_length > 0)
@@ -214,26 +215,28 @@ static void xf_clipboard_free_server_formats(xfClipboard* clipboard)
 	}
 }
 
-static void xf_cliprdr_check_owner(xfClipboard* clipboard)
+static BOOL xf_cliprdr_update_owner(xfClipboard* clipboard)
 {
-	Window owner = 0;
-	xfContext* xfc = NULL;
-
 	WINPR_ASSERT(clipboard);
 
-	xfc = clipboard->xfc;
+	xfContext* xfc = clipboard->xfc;
 	WINPR_ASSERT(xfc);
 
-	if (clipboard->sync)
-	{
-		owner = XGetSelectionOwner(xfc->display, clipboard->clipboard_atom);
+	if (!clipboard->sync)
+		return FALSE;
 
-		if (clipboard->owner != owner)
-		{
-			clipboard->owner = owner;
-			xf_cliprdr_send_client_format_list(clipboard, FALSE);
-		}
-	}
+	Window owner = XGetSelectionOwner(xfc->display, clipboard->clipboard_atom);
+	if (clipboard->owner == owner)
+		return FALSE;
+
+	clipboard->owner = owner;
+	return TRUE;
+}
+
+static void xf_cliprdr_check_owner(xfClipboard* clipboard)
+{
+	if (xf_cliprdr_update_owner(clipboard))
+		xf_cliprdr_send_client_format_list(clipboard, FALSE);
 }
 
 static BOOL xf_cliprdr_is_self_owned(xfClipboard* clipboard)
@@ -580,9 +583,11 @@ static CLIPRDR_FORMAT* xf_cliprdr_get_raw_server_formats(xfClipboard* clipboard,
 	WINPR_ASSERT(xfc);
 
 	*numFormats = 0;
-	LogTagAndXGetWindowProperty(
-	    TAG, xfc->display, clipboard->owner, clipboard->raw_format_list_atom, 0, 4096, False,
-	    clipboard->raw_format_list_atom, &type, &format, &length, &remaining, &data);
+
+	Window owner = XGetSelectionOwner(xfc->display, clipboard->clipboard_atom);
+	LogTagAndXGetWindowProperty(TAG, xfc->display, owner, clipboard->raw_format_list_atom, 0, 4096,
+	                            False, clipboard->raw_format_list_atom, &type, &format, &length,
+	                            &remaining, &data);
 
 	if (data && length > 0 && format == 8 && type == clipboard->raw_format_list_atom)
 	{
@@ -603,6 +608,23 @@ static CLIPRDR_FORMAT* xf_cliprdr_get_raw_server_formats(xfClipboard* clipboard,
 	return formats;
 }
 
+static BOOL xf_cliprdr_should_add_format(const CLIPRDR_FORMAT* formats, size_t count,
+                                         const xfCliprdrFormat* xformat)
+{
+	WINPR_ASSERT(formats);
+
+	if (!xformat)
+		return FALSE;
+
+	for (size_t x = 0; x < count; x++)
+	{
+		const CLIPRDR_FORMAT* format = &formats[x];
+		if (format->formatId == xformat->formatToRequest)
+			return FALSE;
+	}
+	return TRUE;
+}
+
 static CLIPRDR_FORMAT* xf_cliprdr_get_formats_from_targets(xfClipboard* clipboard,
                                                            UINT32* numFormats)
 {
@@ -643,10 +665,18 @@ static CLIPRDR_FORMAT* xf_cliprdr_get_formats_from_targets(xfClipboard* clipboar
 		Atom tatom = ((Atom*)data)[i];
 		const xfCliprdrFormat* format = xf_cliprdr_get_client_format_by_atom(clipboard, tatom);
 
-		if (format)
+		if (xf_cliprdr_should_add_format(formats, *numFormats, format))
 		{
-			formats[*numFormats].formatId = format->formatToRequest;
-			formats[*numFormats].formatName = _strdup(format->formatName);
+			CLIPRDR_FORMAT* cformat = &formats[*numFormats];
+			cformat->formatId = format->formatToRequest;
+			if (format->formatName)
+			{
+				cformat->formatName = _strdup(format->formatName);
+				WINPR_ASSERT(cformat->formatName);
+			}
+			else
+				cformat->formatName = NULL;
+
 			*numFormats += 1;
 		}
 	}
@@ -789,7 +819,7 @@ static UINT xf_cliprdr_send_format_list(xfClipboard* clipboard, const CLIPRDR_FO
 		const CLIPRDR_FORMAT* cpv;
 		CLIPRDR_FORMAT* pv;
 	} cnv = { .cpv = formats };
-	const CLIPRDR_FORMAT_LIST formatList = { .common.msgFlags = CB_RESPONSE_OK,
+	const CLIPRDR_FORMAT_LIST formatList = { .common.msgFlags = 0,
 		                                     .numFormats = numFormats,
 		                                     .formats = cnv.pv,
 		                                     .common.msgType = CB_FORMAT_LIST };
@@ -833,8 +863,8 @@ static void xf_cliprdr_get_requested_targets(xfClipboard* clipboard)
 	xf_cliprdr_free_formats(formats, numFormats);
 }
 
-static void xf_cliprdr_process_requested_data(xfClipboard* clipboard, BOOL hasData, BYTE* data,
-                                              int size)
+static void xf_cliprdr_process_requested_data(xfClipboard* clipboard, BOOL hasData,
+                                              const BYTE* data, size_t size)
 {
 	BOOL bSuccess = 0;
 	UINT32 SrcSize = 0;
@@ -867,7 +897,7 @@ static void xf_cliprdr_process_requested_data(xfClipboard* clipboard, BOOL hasDa
 		case CF_TEXT:
 		case CF_OEMTEXT:
 		case CF_UNICODETEXT:
-			size = strlen((char*)data) + 1;
+			size = strlen((const char*)data) + 1;
 			srcFormatId = format->localFormat;
 			break;
 
@@ -922,14 +952,15 @@ static void xf_cliprdr_process_requested_data(xfClipboard* clipboard, BOOL hasDa
 
 		if (error)
 			WLog_ERR(TAG, "failed to serialize CLIPRDR_FILELIST: 0x%08X", error);
+		else
+		{
+			UINT32 formatId = ClipboardGetFormatId(clipboard->system, mime_uri_list);
+			UINT32 url_size = 0;
 
-		UINT32 formatId = ClipboardGetFormatId(clipboard->system, mime_uri_list);
-		UINT32 url_size = 0;
-		ClipboardLock(clipboard->system);
-		char* url = ClipboardGetData(clipboard->system, formatId, &url_size);
-		ClipboardUnlock(clipboard->system);
-		cliprdr_file_context_update_client_data(clipboard->file, url, url_size);
-		free(url);
+			char* url = ClipboardGetData(clipboard->system, formatId, &url_size);
+			cliprdr_file_context_update_client_data(clipboard->file, url, url_size);
+			free(url);
+		}
 
 		free(file_array);
 	}
@@ -939,6 +970,37 @@ static void xf_cliprdr_process_requested_data(xfClipboard* clipboard, BOOL hasDa
 	free(pDstData);
 }
 
+static BOOL xf_add_input_flags(xfClipboard* clipboard, long mask)
+{
+	WINPR_ASSERT(clipboard);
+
+	xfContext* xfc = clipboard->xfc;
+	WINPR_ASSERT(xfc);
+
+	XWindowAttributes attr = { 0 };
+	XGetWindowAttributes(xfc->display, xfc->drawable, &attr);
+	if ((attr.all_event_masks & mask) == 0)
+		clipboard->event_mask = attr.all_event_masks;
+
+	XSelectInput(xfc->display, xfc->drawable, attr.all_event_masks | mask);
+	return TRUE;
+}
+
+static BOOL xf_restore_input_flags(xfClipboard* clipboard)
+{
+	WINPR_ASSERT(clipboard);
+
+	xfContext* xfc = clipboard->xfc;
+	WINPR_ASSERT(xfc);
+
+	if (clipboard->event_mask != 0)
+	{
+		XSelectInput(xfc->display, xfc->drawable, clipboard->event_mask);
+		clipboard->event_mask = 0;
+	}
+	return TRUE;
+}
+
 static BOOL xf_cliprdr_get_requested_data(xfClipboard* clipboard, Atom target)
 {
 	Atom type = 0;
@@ -988,7 +1050,7 @@ static BOOL xf_cliprdr_get_requested_data(xfClipboard* clipboard, Atom target)
 
 		clipboard->incr_data_length = 0;
 		has_data = TRUE; /* data will be followed in PropertyNotify event */
-		XSelectInput(xfc->display, xfc->drawable, PropertyChangeMask);
+		xf_add_input_flags(clipboard, PropertyChangeMask);
 	}
 	else
 	{
@@ -999,7 +1061,11 @@ static BOOL xf_cliprdr_get_requested_data(xfClipboard* clipboard, Atom target)
 			clipboard->incr_data = NULL;
 			bytes_left = clipboard->incr_data_length;
 			clipboard->incr_data_length = 0;
-			clipboard->incr_starts = 0;
+			clipboard->incr_starts = FALSE;
+
+			/* Restore previous event mask */
+			xf_restore_input_flags(clipboard);
+
 			has_data = TRUE;
 		}
 		else if (LogTagAndXGetWindowProperty(
@@ -1122,7 +1188,8 @@ static void log_selection_event(xfContext* xfc, const XEvent* event)
 			case SelectionClear:
 			{
 				const XSelectionClearEvent* xevent = &event->xselectionclear;
-				char* selection = Safe_XGetAtomName(xfc->display, xevent->selection);
+				char* selection =
+				    Safe_XGetAtomName(_log_cached_ptr, xfc->display, xevent->selection);
 				WLog_Print(_log_cached_ptr, level, "got event %s [selection %s]",
 				           x11_event_string(event->type), selection);
 				XFree(selection);
@@ -1131,9 +1198,10 @@ static void log_selection_event(xfContext* xfc, const XEvent* event)
 			case SelectionNotify:
 			{
 				const XSelectionEvent* xevent = &event->xselection;
-				char* selection = Safe_XGetAtomName(xfc->display, xevent->selection);
-				char* target = Safe_XGetAtomName(xfc->display, xevent->target);
-				char* property = Safe_XGetAtomName(xfc->display, xevent->property);
+				char* selection =
+				    Safe_XGetAtomName(_log_cached_ptr, xfc->display, xevent->selection);
+				char* target = Safe_XGetAtomName(_log_cached_ptr, xfc->display, xevent->target);
+				char* property = Safe_XGetAtomName(_log_cached_ptr, xfc->display, xevent->property);
 				WLog_Print(_log_cached_ptr, level,
 				           "got event %s [selection %s, target %s, property %s]",
 				           x11_event_string(event->type), selection, target, property);
@@ -1145,9 +1213,10 @@ static void log_selection_event(xfContext* xfc, const XEvent* event)
 			case SelectionRequest:
 			{
 				const XSelectionRequestEvent* xevent = &event->xselectionrequest;
-				char* selection = Safe_XGetAtomName(xfc->display, xevent->selection);
-				char* target = Safe_XGetAtomName(xfc->display, xevent->target);
-				char* property = Safe_XGetAtomName(xfc->display, xevent->property);
+				char* selection =
+				    Safe_XGetAtomName(_log_cached_ptr, xfc->display, xevent->selection);
+				char* target = Safe_XGetAtomName(_log_cached_ptr, xfc->display, xevent->target);
+				char* property = Safe_XGetAtomName(_log_cached_ptr, xfc->display, xevent->property);
 				WLog_Print(_log_cached_ptr, level,
 				           "got event %s [selection %s, target %s, property %s]",
 				           x11_event_string(event->type), selection, target, property);
@@ -1159,7 +1228,7 @@ static void log_selection_event(xfContext* xfc, const XEvent* event)
 			case PropertyNotify:
 			{
 				const XPropertyEvent* xevent = &event->xproperty;
-				char* atom = Safe_XGetAtomName(xfc->display, xevent->atom);
+				char* atom = Safe_XGetAtomName(_log_cached_ptr, xfc->display, xevent->atom);
 				WLog_Print(_log_cached_ptr, level, "got event %s [atom %s]",
 				           x11_event_string(event->type), atom);
 				XFree(atom);
@@ -1202,7 +1271,6 @@ void xf_cliprdr_clear_cached_data(xfClipboard* clipboard)
 
 	ClipboardLock(clipboard->system);
 	ClipboardEmpty(clipboard->system);
-	cliprdr_file_context_clear(clipboard->file);
 
 	HashTable_Clear(clipboard->cachedData);
 	HashTable_Clear(clipboard->cachedRawData);
@@ -1675,8 +1743,9 @@ static UINT xf_cliprdr_send_client_format_list(xfClipboard* clipboard, BOOL forc
 	if (clipboard->owner && clipboard->owner != xfc->drawable)
 	{
 		/* Request the owner for TARGETS, and wait for SelectionNotify event */
-		XConvertSelection(xfc->display, clipboard->clipboard_atom, clipboard->targets[1],
-		                  clipboard->property_atom, xfc->drawable, CurrentTime);
+		LogTagAndXConvertSelection(TAG, xfc->display, clipboard->clipboard_atom,
+		                           clipboard->targets[1], clipboard->property_atom, xfc->drawable,
+		                           CurrentTime);
 	}
 
 	xf_cliprdr_free_formats(formats, numFormats);
@@ -1979,8 +2048,8 @@ xf_cliprdr_server_format_data_request(CliprdrClientContext* context,
 	DEBUG_CLIPRDR("requested format 0x%08" PRIx32 " [%s] {local 0x%08" PRIx32 "} [%s]",
 	              format->formatToRequest, ClipboardGetFormatIdString(format->formatToRequest),
 	              format->localFormat, format->formatName);
-	XConvertSelection(xfc->display, clipboard->clipboard_atom, format->atom,
-	                  clipboard->property_atom, xfc->drawable, CurrentTime);
+	LogTagAndXConvertSelection(TAG, xfc->display, clipboard->clipboard_atom, format->atom,
+	                           clipboard->property_atom, xfc->drawable, CurrentTime);
 	XFlush(xfc->display);
 	/* After this point, we expect a SelectionNotify event from the clipboard owner. */
 	return CHANNEL_RC_OK;
@@ -2262,7 +2331,7 @@ xfClipboard* xf_clipboard_new(xfContext* xfc, BOOL relieveFilenameRestriction)
 	if (!selectionAtom)
 		selectionAtom = "CLIPBOARD";
 
-	clipboard->clipboard_atom = XInternAtom(xfc->display, selectionAtom, FALSE);
+	clipboard->clipboard_atom = Logging_XInternAtom(xfc->log, xfc->display, selectionAtom, FALSE);
 
 	if (clipboard->clipboard_atom == None)
 	{
@@ -2271,10 +2340,13 @@ xfClipboard* xf_clipboard_new(xfContext* xfc, BOOL relieveFilenameRestriction)
 	}
 
 	clipboard->timestamp_property_atom =
-	    XInternAtom(xfc->display, "_FREERDP_TIMESTAMP_PROPERTY", FALSE);
-	clipboard->property_atom = XInternAtom(xfc->display, "_FREERDP_CLIPRDR", FALSE);
-	clipboard->raw_transfer_atom = XInternAtom(xfc->display, "_FREERDP_CLIPRDR_RAW", FALSE);
-	clipboard->raw_format_list_atom = XInternAtom(xfc->display, "_FREERDP_CLIPRDR_FORMATS", FALSE);
+	    Logging_XInternAtom(xfc->log, xfc->display, "_FREERDP_TIMESTAMP_PROPERTY", FALSE);
+	clipboard->property_atom =
+	    Logging_XInternAtom(xfc->log, xfc->display, "_FREERDP_CLIPRDR", FALSE);
+	clipboard->raw_transfer_atom =
+	    Logging_XInternAtom(xfc->log, xfc->display, "_FREERDP_CLIPRDR_RAW", FALSE);
+	clipboard->raw_format_list_atom =
+	    Logging_XInternAtom(xfc->log, xfc->display, "_FREERDP_CLIPRDR_FORMATS", FALSE);
 	xf_cliprdr_set_raw_transfer_enabled(clipboard, TRUE);
 	XSelectInput(xfc->display, clipboard->root_window, PropertyChangeMask);
 #ifdef WITH_XFIXES
@@ -2308,11 +2380,11 @@ xfClipboard* xf_clipboard_new(xfContext* xfc, BOOL relieveFilenameRestriction)
 	    "Warning: Using clipboard redirection without XFIXES extension is strongly discouraged!");
 #endif
 	clientFormat = &clipboard->clientFormats[n++];
-	clientFormat->atom = XInternAtom(xfc->display, "_FREERDP_RAW", False);
+	clientFormat->atom = Logging_XInternAtom(xfc->log, xfc->display, "_FREERDP_RAW", False);
 	clientFormat->localFormat = clientFormat->formatToRequest = CF_RAW;
 
 	clientFormat = &clipboard->clientFormats[n++];
-	clientFormat->atom = XInternAtom(xfc->display, "UTF8_STRING", False);
+	clientFormat->atom = Logging_XInternAtom(xfc->log, xfc->display, "UTF8_STRING", False);
 	clientFormat->formatToRequest = CF_UNICODETEXT;
 	clientFormat->localFormat = ClipboardGetFormatId(xfc->clipboard->system, mime_text_plain);
 
@@ -2322,7 +2394,7 @@ xfClipboard* xf_clipboard_new(xfContext* xfc, BOOL relieveFilenameRestriction)
 	clientFormat->localFormat = ClipboardGetFormatId(xfc->clipboard->system, mime_text_plain);
 
 	clientFormat = &clipboard->clientFormats[n++];
-	clientFormat->atom = XInternAtom(xfc->display, mime_tiff, False);
+	clientFormat->atom = Logging_XInternAtom(xfc->log, xfc->display, mime_tiff, False);
 	clientFormat->formatToRequest = clientFormat->localFormat = CF_TIFF;
 
 	for (size_t x = 0; x < ARRAYSIZE(mime_bitmap); x++)
@@ -2338,7 +2410,7 @@ xfClipboard* xf_clipboard_new(xfContext* xfc, BOOL relieveFilenameRestriction)
 		WLog_DBG(TAG, "register local bitmap format %s [0x%08" PRIx32 "]", mime_bmp, format);
 		clientFormat = &clipboard->clientFormats[n++];
 		clientFormat->localFormat = format;
-		clientFormat->atom = XInternAtom(xfc->display, mime_bmp, False);
+		clientFormat->atom = Logging_XInternAtom(xfc->log, xfc->display, mime_bmp, False);
 		clientFormat->formatToRequest = CF_DIB;
 	}
 
@@ -2355,12 +2427,12 @@ xfClipboard* xf_clipboard_new(xfContext* xfc, BOOL relieveFilenameRestriction)
 		WLog_DBG(TAG, "register local bitmap format %s [0x%08" PRIx32 "]", mime_bmp, format);
 		clientFormat = &clipboard->clientFormats[n++];
 		clientFormat->localFormat = format;
-		clientFormat->atom = XInternAtom(xfc->display, mime_bmp, False);
+		clientFormat->atom = Logging_XInternAtom(xfc->log, xfc->display, mime_bmp, False);
 		clientFormat->formatToRequest = CF_DIB;
 	}
 
 	clientFormat = &clipboard->clientFormats[n++];
-	clientFormat->atom = XInternAtom(xfc->display, mime_html, False);
+	clientFormat->atom = Logging_XInternAtom(xfc->log, xfc->display, mime_html, False);
 	clientFormat->formatToRequest = ClipboardGetFormatId(xfc->clipboard->system, type_HtmlFormat);
 	clientFormat->localFormat = ClipboardGetFormatId(xfc->clipboard->system, mime_html);
 	clientFormat->formatName = _strdup(type_HtmlFormat);
@@ -2381,7 +2453,7 @@ xfClipboard* xf_clipboard_new(xfContext* xfc, BOOL relieveFilenameRestriction)
 	if (uid)
 	{
 		cliprdr_file_context_set_locally_available(clipboard->file, TRUE);
-		clientFormat->atom = XInternAtom(xfc->display, mime_uri_list, False);
+		clientFormat->atom = Logging_XInternAtom(xfc->log, xfc->display, mime_uri_list, False);
 		clientFormat->localFormat = uid;
 		clientFormat->formatToRequest = fgid;
 		clientFormat->formatName = _strdup(type_FileGroupDescriptorW);
@@ -2396,7 +2468,8 @@ xfClipboard* xf_clipboard_new(xfContext* xfc, BOOL relieveFilenameRestriction)
 	if (gid != 0)
 	{
 		cliprdr_file_context_set_locally_available(clipboard->file, TRUE);
-		clientFormat->atom = XInternAtom(xfc->display, mime_gnome_copied_files, False);
+		clientFormat->atom =
+		    Logging_XInternAtom(xfc->log, xfc->display, mime_gnome_copied_files, False);
 		clientFormat->localFormat = gid;
 		clientFormat->formatToRequest = fgid;
 		clientFormat->formatName = _strdup(type_FileGroupDescriptorW);
@@ -2411,7 +2484,8 @@ xfClipboard* xf_clipboard_new(xfContext* xfc, BOOL relieveFilenameRestriction)
 	if (mid != 0)
 	{
 		cliprdr_file_context_set_locally_available(clipboard->file, TRUE);
-		clientFormat->atom = XInternAtom(xfc->display, mime_mate_copied_files, False);
+		clientFormat->atom =
+		    Logging_XInternAtom(xfc->log, xfc->display, mime_mate_copied_files, False);
 		clientFormat->localFormat = mid;
 		clientFormat->formatToRequest = fgid;
 		clientFormat->formatName = _strdup(type_FileGroupDescriptorW);
@@ -2421,10 +2495,10 @@ xfClipboard* xf_clipboard_new(xfContext* xfc, BOOL relieveFilenameRestriction)
 	}
 
 	clipboard->numClientFormats = n;
-	clipboard->targets[0] = XInternAtom(xfc->display, "TIMESTAMP", FALSE);
-	clipboard->targets[1] = XInternAtom(xfc->display, "TARGETS", FALSE);
+	clipboard->targets[0] = Logging_XInternAtom(xfc->log, xfc->display, "TIMESTAMP", FALSE);
+	clipboard->targets[1] = Logging_XInternAtom(xfc->log, xfc->display, "TARGETS", FALSE);
 	clipboard->numTargets = 2;
-	clipboard->incr_atom = XInternAtom(xfc->display, "INCR", FALSE);
+	clipboard->incr_atom = Logging_XInternAtom(xfc->log, xfc->display, "INCR", FALSE);
 
 	if (relieveFilenameRestriction)
 	{
