@@ -23,7 +23,10 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
+#include <string.h>
+
 #include <winpr/assert.h>
+#include <winpr/path.h>
 
 #include <freerdp/log.h>
 #include <freerdp/locale/keyboard.h>
@@ -35,16 +38,20 @@
 #include "xf_input.h"
 #include "xf_gfx.h"
 #include "xf_graphics.h"
+#include "xf_utils.h"
 
 #include "xf_event.h"
 
 #define TAG CLIENT_TAG("x11")
 
 #define CLAMP_COORDINATES(x, y) \
-	if (x < 0)                  \
-		x = 0;                  \
-	if (y < 0)                  \
-	y = 0
+	do                          \
+	{                           \
+		if ((x) < 0)            \
+			(x) = 0;            \
+		if ((y) < 0)            \
+			(y) = 0;            \
+	} while (0)
 
 const char* x11_event_string(int event)
 {
@@ -166,14 +173,28 @@ const char* x11_event_string(int event)
 	} while (0)
 #endif
 
+static BOOL xf_action_script_append(xfContext* xfc, const char* buffer, size_t size, void* user,
+                                    const char* what, const char* arg)
+{
+	WINPR_ASSERT(xfc);
+	WINPR_UNUSED(what);
+	WINPR_UNUSED(arg);
+
+	if (buffer || (size == 0))
+		return TRUE;
+
+	if (!ArrayList_Append(xfc->xevents, buffer))
+	{
+		ArrayList_Clear(xfc->xevents);
+		return FALSE;
+	}
+	return TRUE;
+}
+
 BOOL xf_event_action_script_init(xfContext* xfc)
 {
 	wObject* obj = NULL;
-	FILE* actionScript = NULL;
-	char buffer[1024] = { 0 };
-	char command[1024] = { 0 };
 	const rdpSettings* settings = NULL;
-	const char* ActionScript = NULL;
 
 	WINPR_ASSERT(xfc);
 
@@ -189,28 +210,10 @@ BOOL xf_event_action_script_init(xfContext* xfc)
 	WINPR_ASSERT(obj);
 	obj->fnObjectNew = winpr_ObjectStringClone;
 	obj->fnObjectFree = winpr_ObjectStringFree;
-	ActionScript = freerdp_settings_get_string(settings, FreeRDP_ActionScript);
-	sprintf_s(command, sizeof(command), "%s xevent", ActionScript);
-	actionScript = popen(command, "r");
 
-	if (!actionScript)
+	if (!run_action_script(xfc, "xevent", NULL, xf_action_script_append, NULL))
 		return FALSE;
 
-	while (fgets(buffer, sizeof(buffer), actionScript))
-	{
-		char* context = NULL;
-		strtok_s(buffer, "\n", &context);
-
-		if (!buffer || !ArrayList_Append(xfc->xevents, buffer))
-		{
-			pclose(actionScript);
-			ArrayList_Free(xfc->xevents);
-			xfc->xevents = NULL;
-			return FALSE;
-		}
-	}
-
-	pclose(actionScript);
 	return TRUE;
 }
 
@@ -223,16 +226,59 @@ void xf_event_action_script_free(xfContext* xfc)
 	}
 }
 
+static BOOL action_script_run(xfContext* xfc, const char* buffer, size_t size, void* user,
+                              const char* what, const char* arg)
+{
+	WINPR_UNUSED(xfc);
+	WINPR_UNUSED(what);
+	WINPR_UNUSED(arg);
+	WINPR_ASSERT(user);
+	int* pstatus = user;
+
+	if (size == 0)
+	{
+		WLog_WARN(TAG, "ActionScript xevent: script did not return data");
+		return FALSE;
+	}
+
+	if (winpr_PathFileExists(buffer))
+	{
+		char* cmd = NULL;
+		size_t cmdlen = 0;
+		winpr_asprintf(&cmd, &cmdlen, "%s %s %s", buffer, what, arg);
+		if (!cmd)
+			return FALSE;
+
+		FILE* fp = popen(cmd, "w");
+		free(cmd);
+		if (!fp)
+		{
+			WLog_ERR(TAG, "Failed to execute '%s'", buffer);
+			return FALSE;
+		}
+
+		*pstatus = pclose(fp);
+		if (*pstatus < 0)
+		{
+			WLog_ERR(TAG, "Command '%s' returned %d", buffer, *pstatus);
+			return FALSE;
+		}
+	}
+	else
+	{
+		WLog_WARN(TAG, "ActionScript xevent: No such file '%s'", buffer);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static BOOL xf_event_execute_action_script(xfContext* xfc, const XEvent* event)
 {
-	int count = 0;
+	size_t count = 0;
 	char* name = NULL;
-	FILE* actionScript = NULL;
 	BOOL match = FALSE;
 	const char* xeventName = NULL;
-	const char* ActionScript = NULL;
-	char buffer[1024] = { 0 };
-	char command[1024] = { 0 };
 
 	if (!xfc->actionScriptExists || !xfc->xevents || !xfc->window)
 		return FALSE;
@@ -243,7 +289,7 @@ static BOOL xf_event_execute_action_script(xfContext* xfc, const XEvent* event)
 	xeventName = x11_event_string(event->type);
 	count = ArrayList_Count(xfc->xevents);
 
-	for (int index = 0; index < count; index++)
+	for (size_t index = 0; index < count; index++)
 	{
 		name = (char*)ArrayList_GetItem(xfc->xevents, index);
 
@@ -257,21 +303,12 @@ static BOOL xf_event_execute_action_script(xfContext* xfc, const XEvent* event)
 	if (!match)
 		return FALSE;
 
-	ActionScript = freerdp_settings_get_string(xfc->common.context.settings, FreeRDP_ActionScript);
-	sprintf_s(command, sizeof(command), "%s xevent %s %lu", ActionScript, xeventName,
-	          (unsigned long)xfc->window->handle);
-	actionScript = popen(command, "r");
-
-	if (!actionScript)
+	char command[2048] = { 0 };
+	char arg[2048] = { 0 };
+	_snprintf(command, sizeof(command), "xevent %s", xeventName);
+	_snprintf(arg, sizeof(arg), "%lu", (unsigned long)xfc->window->handle);
+	if (!run_action_script(xfc, command, arg, action_script_run, NULL))
 		return FALSE;
-
-	while (fgets(buffer, sizeof(buffer), actionScript))
-	{
-		char* context = NULL;
-		strtok_s(buffer, "\n", &context);
-	}
-
-	pclose(actionScript);
 	return TRUE;
 }
 
@@ -447,12 +484,13 @@ static BOOL xf_event_MotionNotify(xfContext* xfc, const XMotionEvent* event, BOO
 {
 	WINPR_ASSERT(xfc);
 
-	if (xfc->xi_event)
-		return TRUE;
-
 	if (xfc->window)
 		xf_floatbar_set_root_y(xfc->window->floatbar, event->y);
 
+	if (xfc->xi_event ||
+	    (xfc->common.mouse_grabbed && freerdp_client_use_relative_mouse_events(&xfc->common)))
+		return TRUE;
+
 	return xf_generic_MotionNotify(xfc, event->x, event->y, event->state, event->window, app);
 }
 
@@ -556,7 +594,8 @@ static BOOL xf_event_ButtonPress(xfContext* xfc, const XButtonEvent* event, BOOL
 {
 	xf_grab_mouse(xfc);
 
-	if (xfc->xi_event)
+	if (xfc->xi_event ||
+	    (xfc->common.mouse_grabbed && freerdp_client_use_relative_mouse_events(&xfc->common)))
 		return TRUE;
 	return xf_generic_ButtonEvent(xfc, event->x, event->y, event->button, event->window, app, TRUE);
 }
@@ -565,7 +604,8 @@ static BOOL xf_event_ButtonRelease(xfContext* xfc, const XButtonEvent* event, BO
 {
 	xf_grab_mouse(xfc);
 
-	if (xfc->xi_event)
+	if (xfc->xi_event ||
+	    (xfc->common.mouse_grabbed && freerdp_client_use_relative_mouse_events(&xfc->common)))
 		return TRUE;
 	return xf_generic_ButtonEvent(xfc, event->x, event->y, event->button, event->window, app,
 	                              FALSE);
@@ -645,6 +685,8 @@ static BOOL xf_event_FocusIn(xfContext* xfc, const XFocusInEvent* event, BOOL ap
 	 * if the WM decided to use an alternate event order */
 	if (!app)
 		xf_keyboard_release_all_keypress(xfc);
+	else
+		xf_rail_send_activate(xfc, event->window, TRUE);
 
 	xf_pointer_update_scale(xfc);
 
@@ -673,6 +715,8 @@ static BOOL xf_event_FocusOut(xfContext* xfc, const XFocusOutEvent* event, BOOL
 		XUngrabKeyboard(xfc->display, CurrentTime);
 
 	xf_keyboard_release_all_keypress(xfc);
+	if (app)
+		xf_rail_send_activate(xfc, event->window, FALSE);
 
 	return TRUE;
 }
@@ -946,14 +990,16 @@ static BOOL xf_event_PropertyNotify(xfContext* xfc, const XPropertyEvent* event,
 				for (unsigned long i = 0; i < nitems; i++)
 				{
 					if ((Atom)((UINT16**)prop)[i] ==
-					    XInternAtom(xfc->display, "_NET_WM_STATE_MAXIMIZED_VERT", False))
+					    Logging_XInternAtom(xfc->log, xfc->display, "_NET_WM_STATE_MAXIMIZED_VERT",
+					                        False))
 					{
 						if (appWindow)
 							appWindow->maxVert = TRUE;
 					}
 
 					if ((Atom)((UINT16**)prop)[i] ==
-					    XInternAtom(xfc->display, "_NET_WM_STATE_MAXIMIZED_HORZ", False))
+					    Logging_XInternAtom(xfc->log, xfc->display, "_NET_WM_STATE_MAXIMIZED_HORZ",
+					                        False))
 					{
 						if (appWindow)
 							appWindow->maxHorz = TRUE;
@@ -972,7 +1018,7 @@ static BOOL xf_event_PropertyNotify(xfContext* xfc, const XPropertyEvent* event,
 			if (status)
 			{
 				/* If the window is in the iconic state */
-				if (((UINT32)*prop == 3))
+				if (((UINT32)*prop == 3) && !IsGnome())
 				{
 					minimized = TRUE;
 					if (appWindow)
