@@ -40,15 +40,14 @@
 
 struct S_FREERDP_DSP_CONTEXT
 {
-	AUDIO_FORMAT format;
+	FREERDP_DSP_COMMON_CONTEXT common;
 
 	BOOL isOpen;
-	BOOL encoder;
 
 	UINT32 bufferedSamples;
 
 	enum AVCodecID id;
-	AVCodec* codec;
+	const AVCodec* codec;
 	AVCodecContext* context;
 	AVFrame* frame;
 	AVFrame* resampled;
@@ -59,7 +58,6 @@ struct S_FREERDP_DSP_CONTEXT
 #else
 	AVAudioResampleContext* rcontext;
 #endif
-	wStream* channelmix;
 };
 
 static BOOL ffmpeg_codec_is_filtered(enum AVCodecID id, BOOL encoder)
@@ -92,7 +90,7 @@ static BOOL ffmpeg_codec_is_filtered(enum AVCodecID id, BOOL encoder)
 	}
 }
 
-static enum AVCodecID ffmpeg_get_avcodec(const AUDIO_FORMAT* format)
+static enum AVCodecID ffmpeg_get_avcodec(const AUDIO_FORMAT* WINPR_RESTRICT format)
 {
 	if (!format)
 		return AV_CODEC_ID_NONE;
@@ -144,7 +142,7 @@ static enum AVCodecID ffmpeg_get_avcodec(const AUDIO_FORMAT* format)
 	}
 }
 
-static int ffmpeg_sample_format(const AUDIO_FORMAT* format)
+static int ffmpeg_sample_format(const AUDIO_FORMAT* WINPR_RESTRICT format)
 {
 	switch (format->wFormatTag)
 	{
@@ -184,7 +182,7 @@ static int ffmpeg_sample_format(const AUDIO_FORMAT* format)
 	}
 }
 
-static void ffmpeg_close_context(FREERDP_DSP_CONTEXT* context)
+static void ffmpeg_close_context(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context)
 {
 	if (context)
 	{
@@ -223,26 +221,23 @@ static void ffmpeg_close_context(FREERDP_DSP_CONTEXT* context)
 	}
 }
 
-static BOOL ffmpeg_open_context(FREERDP_DSP_CONTEXT* context)
+static BOOL ffmpeg_open_context(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context)
 {
-	int ret;
+	int ret = 0;
 
 	if (!context || context->isOpen)
 		return FALSE;
 
-	const AUDIO_FORMAT* format = &context->format;
+	const AUDIO_FORMAT* format = &context->common.format;
 
 	if (!format)
 		return FALSE;
-#if LIBAVUTIL_VERSION_INT < AV_VERSION_INT(57, 28, 100)
-	const int layout = av_get_default_channel_layout(format->nChannels);
-#endif
 	context->id = ffmpeg_get_avcodec(format);
 
-	if (ffmpeg_codec_is_filtered(context->id, context->encoder))
+	if (ffmpeg_codec_is_filtered(context->id, context->common.encoder))
 		goto fail;
 
-	if (context->encoder)
+	if (context->common.encoder)
 		context->codec = avcodec_find_encoder(context->id);
 	else
 		context->codec = avcodec_find_decoder(context->id);
@@ -272,13 +267,15 @@ static BOOL ffmpeg_open_context(FREERDP_DSP_CONTEXT* context)
 
 	context->context->max_b_frames = 1;
 	context->context->delay = 0;
-#if LIBAVUTIL_VERSION_INT < AV_VERSION_INT(57, 28, 100)
-	context->context->channels = format->nChannels;
-	context->context->channel_layout = layout;
-#else
+
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 28, 100)
 	av_channel_layout_default(&context->context->ch_layout, format->nChannels);
+#else
+	context->context->channels = format->nChannels;
+	const int64_t layout = av_get_default_channel_layout(format->nChannels);
+	context->context->channel_layout = layout;
 #endif
-	context->context->sample_rate = format->nSamplesPerSec;
+	context->context->sample_rate = (int)format->nSamplesPerSec;
 	context->context->block_align = format->nBlockAlign;
 	context->context->bit_rate = format->nAvgBytesPerSec * 8;
 	context->context->sample_fmt = ffmpeg_sample_format(format);
@@ -320,16 +317,17 @@ static BOOL ffmpeg_open_context(FREERDP_DSP_CONTEXT* context)
 	if (!context->rcontext)
 		goto fail;
 
-#if LIBAVUTIL_VERSION_INT < AV_VERSION_INT(57, 28, 100)
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 28, 100)
+	av_channel_layout_default(&context->frame->ch_layout, format->nChannels);
+#else
 	context->frame->channel_layout = layout;
 	context->frame->channels = format->nChannels;
-#else
-	av_channel_layout_default(&context->frame->ch_layout, format->nChannels);
 #endif
-	context->frame->sample_rate = format->nSamplesPerSec;
+	WINPR_ASSERT(format->nSamplesPerSec <= INT_MAX);
+	context->frame->sample_rate = (int)format->nSamplesPerSec;
 	context->frame->format = AV_SAMPLE_FMT_S16;
 
-	if (context->encoder)
+	if (context->common.encoder)
 	{
 		context->resampled->format = context->context->sample_fmt;
 		context->resampled->sample_rate = context->context->sample_rate;
@@ -337,23 +335,27 @@ static BOOL ffmpeg_open_context(FREERDP_DSP_CONTEXT* context)
 	else
 	{
 		context->resampled->format = AV_SAMPLE_FMT_S16;
-		context->resampled->sample_rate = format->nSamplesPerSec;
+
+		WINPR_ASSERT(format->nSamplesPerSec <= INT_MAX);
+		context->resampled->sample_rate = (int)format->nSamplesPerSec;
 	}
 
-#if LIBAVUTIL_VERSION_INT < AV_VERSION_INT(57, 28, 100)
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 28, 100)
+	av_channel_layout_default(&context->resampled->ch_layout, format->nChannels);
+#else
 	context->resampled->channel_layout = layout;
 	context->resampled->channels = format->nChannels;
-#else
-	av_channel_layout_default(&context->resampled->ch_layout, format->nChannels);
 #endif
 
 	if (context->context->frame_size > 0)
 	{
-#if LIBAVUTIL_VERSION_INT < AV_VERSION_INT(57, 28, 100)
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 28, 100)
+		ret = av_channel_layout_copy(&context->buffered->ch_layout, &context->resampled->ch_layout);
+		if (ret != 0)
+			goto fail;
+#else
 		context->buffered->channel_layout = context->resampled->channel_layout;
 		context->buffered->channels = context->resampled->channels;
-#else
-		av_channel_layout_copy(&context->buffered->ch_layout, &context->resampled->ch_layout);
 #endif
 		context->buffered->format = context->resampled->format;
 		context->buffered->nb_samples = context->context->frame_size;
@@ -370,9 +372,10 @@ fail:
 }
 
 #if defined(SWRESAMPLE_FOUND)
-static BOOL ffmpeg_resample_frame(SwrContext* context, AVFrame* in, AVFrame* out)
+static BOOL ffmpeg_resample_frame(SwrContext* WINPR_RESTRICT context, AVFrame* WINPR_RESTRICT in,
+                                  AVFrame* WINPR_RESTRICT out)
 {
-	int ret;
+	int ret = 0;
 
 	if (!swr_is_initialized(context))
 	{
@@ -401,7 +404,8 @@ static BOOL ffmpeg_resample_frame(SwrContext* context, AVFrame* in, AVFrame* out
 	return TRUE;
 }
 #else
-static BOOL ffmpeg_resample_frame(AVAudioResampleContext* context, AVFrame* in, AVFrame* out)
+static BOOL ffmpeg_resample_frame(AVAudioResampleContext* WINPR_RESTRICT context,
+                                  AVFrame* WINPR_RESTRICT in, AVFrame* WINPR_RESTRICT out)
 {
 	int ret;
 
@@ -433,8 +437,8 @@ static BOOL ffmpeg_resample_frame(AVAudioResampleContext* context, AVFrame* in,
 }
 #endif
 
-static BOOL ffmpeg_encode_frame(AVCodecContext* context, AVFrame* in, AVPacket* packet,
-                                wStream* out)
+static BOOL ffmpeg_encode_frame(AVCodecContext* WINPR_RESTRICT context, AVFrame* WINPR_RESTRICT in,
+                                AVPacket* WINPR_RESTRICT packet, wStream* WINPR_RESTRICT out)
 {
 	if (in->format == AV_SAMPLE_FMT_FLTP)
 	{
@@ -487,34 +491,40 @@ static BOOL ffmpeg_encode_frame(AVCodecContext* context, AVFrame* in, AVPacket*
 			return FALSE;
 		}
 
-		if (!Stream_EnsureRemainingCapacity(out, packet->size))
+		WINPR_ASSERT(packet->size >= 0);
+		if (!Stream_EnsureRemainingCapacity(out, (size_t)packet->size))
 			return FALSE;
 
-		Stream_Write(out, packet->data, packet->size);
+		Stream_Write(out, packet->data, (size_t)packet->size);
 		av_packet_unref(packet);
 	}
 
 	return TRUE;
 }
 
-static BOOL ffmpeg_fill_frame(AVFrame* frame, const AUDIO_FORMAT* inputFormat, const BYTE* data,
-                              size_t size)
+static BOOL ffmpeg_fill_frame(AVFrame* WINPR_RESTRICT frame,
+                              const AUDIO_FORMAT* WINPR_RESTRICT inputFormat,
+                              const BYTE* WINPR_RESTRICT data, size_t size)
 {
-	int ret, bpp;
+	int ret = 0;
 #if LIBAVUTIL_VERSION_INT < AV_VERSION_INT(57, 28, 100)
 	frame->channels = inputFormat->nChannels;
 	frame->channel_layout = av_get_default_channel_layout(frame->channels);
 #else
 	av_channel_layout_default(&frame->ch_layout, inputFormat->nChannels);
 #endif
-	frame->sample_rate = inputFormat->nSamplesPerSec;
+	WINPR_ASSERT(inputFormat->nSamplesPerSec <= INT_MAX);
+	frame->sample_rate = (int)inputFormat->nSamplesPerSec;
 	frame->format = ffmpeg_sample_format(inputFormat);
 
-	bpp = av_get_bytes_per_sample(frame->format);
-	frame->nb_samples = size / inputFormat->nChannels / bpp;
+	const int bpp = av_get_bytes_per_sample(frame->format);
+	WINPR_ASSERT(bpp >= 0);
+	WINPR_ASSERT(size <= INT_MAX);
+	const size_t nb_samples = size / inputFormat->nChannels / (size_t)bpp;
+	frame->nb_samples = (int)nb_samples;
 
-	if ((ret = avcodec_fill_audio_frame(frame, inputFormat->nChannels, frame->format, data, size,
-	                                    1)) < 0)
+	if ((ret = avcodec_fill_audio_frame(frame, inputFormat->nChannels, frame->format, data,
+	                                    (int)size, 1)) < 0)
 	{
 		const char* err = av_err2str(ret);
 		WLog_ERR(TAG, "Error during audio frame fill %s [%d]", err, ret);
@@ -524,14 +534,15 @@ static BOOL ffmpeg_fill_frame(AVFrame* frame, const AUDIO_FORMAT* inputFormat, c
 	return TRUE;
 }
 #if defined(SWRESAMPLE_FOUND)
-static BOOL ffmpeg_decode(AVCodecContext* dec_ctx, AVPacket* pkt, AVFrame* frame,
-                          SwrContext* resampleContext, AVFrame* resampled, wStream* out)
+static BOOL ffmpeg_decode(AVCodecContext* WINPR_RESTRICT dec_ctx, AVPacket* WINPR_RESTRICT pkt,
+                          AVFrame* WINPR_RESTRICT frame, SwrContext* WINPR_RESTRICT resampleContext,
+                          AVFrame* WINPR_RESTRICT resampled, wStream* WINPR_RESTRICT out)
 #else
 static BOOL ffmpeg_decode(AVCodecContext* dec_ctx, AVPacket* pkt, AVFrame* frame,
                           AVAudioResampleContext* resampleContext, AVFrame* resampled, wStream* out)
 #endif
 {
-	int ret;
+	int ret = 0;
 	/* send the packet with the compressed data to the decoder */
 	ret = avcodec_send_packet(dec_ctx, pkt);
 
@@ -596,13 +607,17 @@ static BOOL ffmpeg_decode(AVCodecContext* dec_ctx, AVPacket* pkt, AVFrame* frame
 		}
 
 		{
-#if LIBAVUTIL_VERSION_INT < AV_VERSION_INT(57, 28, 100)
-			const size_t channels = resampled->channels;
+
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 28, 100)
+			WINPR_ASSERT(resampled->ch_layout.nb_channels >= 0);
+			const size_t nrchannels = (size_t)resampled->ch_layout.nb_channels;
 #else
-			const size_t channels = resampled->ch_layout.nb_channels;
+			const size_t nrchannels = resampled->channels;
 #endif
-			const size_t data_size = channels * resampled->nb_samples * 2;
-			Stream_EnsureRemainingCapacity(out, data_size);
+			WINPR_ASSERT(resampled->nb_samples >= 0);
+			const size_t data_size = nrchannels * (size_t)resampled->nb_samples * 2ull;
+			if (!Stream_EnsureRemainingCapacity(out, data_size))
+				return FALSE;
 			Stream_Write(out, resampled->data[0], data_size);
 		}
 	}
@@ -610,7 +625,7 @@ static BOOL ffmpeg_decode(AVCodecContext* dec_ctx, AVPacket* pkt, AVFrame* frame
 	return TRUE;
 }
 
-BOOL freerdp_dsp_ffmpeg_supports_format(const AUDIO_FORMAT* format, BOOL encode)
+BOOL freerdp_dsp_ffmpeg_supports_format(const AUDIO_FORMAT* WINPR_RESTRICT format, BOOL encode)
 {
 	enum AVCodecID id = ffmpeg_get_avcodec(format);
 
@@ -625,23 +640,23 @@ BOOL freerdp_dsp_ffmpeg_supports_format(const AUDIO_FORMAT* format, BOOL encode)
 
 FREERDP_DSP_CONTEXT* freerdp_dsp_ffmpeg_context_new(BOOL encode)
 {
-	FREERDP_DSP_CONTEXT* context;
+	FREERDP_DSP_CONTEXT* context = NULL;
 #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(58, 10, 100)
 	avcodec_register_all();
 #endif
 	context = calloc(1, sizeof(FREERDP_DSP_CONTEXT));
 
 	if (!context)
-		return NULL;
+		goto fail;
+
+	if (!freerdp_dsp_common_context_init(&context->common, encode))
+		goto fail;
 
-	context->channelmix = Stream_New(NULL, 1024);
-	if (!context->channelmix)
-	{
-		freerdp_dsp_ffmpeg_context_free(context);
-		return NULL;
-	}
-	context->encoder = encode;
 	return context;
+
+fail:
+	freerdp_dsp_ffmpeg_context_free(context);
+	return NULL;
 }
 
 void freerdp_dsp_ffmpeg_context_free(FREERDP_DSP_CONTEXT* context)
@@ -649,29 +664,30 @@ void freerdp_dsp_ffmpeg_context_free(FREERDP_DSP_CONTEXT* context)
 	if (context)
 	{
 		ffmpeg_close_context(context);
-		Stream_Free(context->channelmix, TRUE);
+		freerdp_dsp_common_context_uninit(&context->common);
 		free(context);
 	}
 }
 
-BOOL freerdp_dsp_ffmpeg_context_reset(FREERDP_DSP_CONTEXT* context,
-                                      const AUDIO_FORMAT* targetFormat)
+BOOL freerdp_dsp_ffmpeg_context_reset(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context,
+                                      const AUDIO_FORMAT* WINPR_RESTRICT targetFormat)
 {
 	if (!context || !targetFormat)
 		return FALSE;
 
 	ffmpeg_close_context(context);
-	context->format = *targetFormat;
+	context->common.format = *targetFormat;
 	return ffmpeg_open_context(context);
 }
 
-static BOOL freerdp_dsp_channel_mix(FREERDP_DSP_CONTEXT* context, const BYTE* src, size_t size,
-                                    const AUDIO_FORMAT* srcFormat, const BYTE** data,
-                                    size_t* length, AUDIO_FORMAT* dstFormat)
+static BOOL freerdp_dsp_channel_mix(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context,
+                                    const BYTE* WINPR_RESTRICT src, size_t size,
+                                    const AUDIO_FORMAT* WINPR_RESTRICT srcFormat,
+                                    const BYTE** WINPR_RESTRICT data, size_t* WINPR_RESTRICT length,
+                                    AUDIO_FORMAT* WINPR_RESTRICT dstFormat)
 {
-	UINT32 bpp;
-	size_t samples;
-	size_t x, y;
+	UINT32 bpp = 0;
+	size_t samples = 0;
 
 	if (!context || !data || !length || !dstFormat)
 		return FALSE;
@@ -683,42 +699,43 @@ static BOOL freerdp_dsp_channel_mix(FREERDP_DSP_CONTEXT* context, const BYTE* sr
 	samples = size / bpp / srcFormat->nChannels;
 
 	*dstFormat = *srcFormat;
-	if (context->format.nChannels == srcFormat->nChannels)
+	if (context->common.format.nChannels == srcFormat->nChannels)
 	{
 		*data = src;
 		*length = size;
 		return TRUE;
 	}
 
-	Stream_SetPosition(context->channelmix, 0);
+	Stream_SetPosition(context->common.channelmix, 0);
 
 	/* Destination has more channels than source */
-	if (context->format.nChannels > srcFormat->nChannels)
+	if (context->common.format.nChannels > srcFormat->nChannels)
 	{
 		switch (srcFormat->nChannels)
 		{
 			case 1:
-				if (!Stream_EnsureCapacity(context->channelmix, size * 2))
+				if (!Stream_EnsureCapacity(context->common.channelmix, size * 2))
 					return FALSE;
 
-				for (x = 0; x < samples; x++)
+				for (size_t x = 0; x < samples; x++)
 				{
-					for (y = 0; y < bpp; y++)
-						Stream_Write_UINT8(context->channelmix, src[x * bpp + y]);
+					for (size_t y = 0; y < bpp; y++)
+						Stream_Write_UINT8(context->common.channelmix, src[x * bpp + y]);
 
-					for (y = 0; y < bpp; y++)
-						Stream_Write_UINT8(context->channelmix, src[x * bpp + y]);
+					for (size_t y = 0; y < bpp; y++)
+						Stream_Write_UINT8(context->common.channelmix, src[x * bpp + y]);
 				}
 
-				Stream_SealLength(context->channelmix);
-				*data = Stream_Buffer(context->channelmix);
-				*length = Stream_Length(context->channelmix);
+				Stream_SealLength(context->common.channelmix);
+				*data = Stream_Buffer(context->common.channelmix);
+				*length = Stream_Length(context->common.channelmix);
 				dstFormat->nChannels = 2;
 				return TRUE;
 
 			case 2:  /* We only support stereo, so we can not handle this case. */
 			default: /* Unsupported number of channels */
-				WLog_WARN(TAG, "unsupported source channel count %" PRIu16, srcFormat->nChannels);
+				WLog_WARN(TAG, "[%s] unsuported source channel count %" PRIu16, __func__,
+				          srcFormat->nChannels);
 				return FALSE;
 		}
 	}
@@ -727,39 +744,41 @@ static BOOL freerdp_dsp_channel_mix(FREERDP_DSP_CONTEXT* context, const BYTE* sr
 	switch (srcFormat->nChannels)
 	{
 		case 2:
-			if (!Stream_EnsureCapacity(context->channelmix, size / 2))
+			if (!Stream_EnsureCapacity(context->common.channelmix, size / 2))
 				return FALSE;
 
 			/* Simply drop second channel.
 			 * TODO: Calculate average */
-			for (x = 0; x < samples; x++)
+			for (size_t x = 0; x < samples; x++)
 			{
-				for (y = 0; y < bpp; y++)
-					Stream_Write_UINT8(context->channelmix, src[2 * x * bpp + y]);
+				for (size_t y = 0; y < bpp; y++)
+					Stream_Write_UINT8(context->common.channelmix, src[2 * x * bpp + y]);
 			}
 
-			Stream_SealLength(context->channelmix);
-			*data = Stream_Buffer(context->channelmix);
-			*length = Stream_Length(context->channelmix);
+			Stream_SealLength(context->common.channelmix);
+			*data = Stream_Buffer(context->common.channelmix);
+			*length = Stream_Length(context->common.channelmix);
 			dstFormat->nChannels = 1;
 			return TRUE;
 
 		case 1:  /* Invalid, do we want to use a 0 channel sound? */
 		default: /* Unsupported number of channels */
-			WLog_WARN(TAG, "unsupported channel count %" PRIu16, srcFormat->nChannels);
+			WLog_WARN(TAG, "[%s] unsuported channel count %" PRIu16, __func__,
+			          srcFormat->nChannels);
 			return FALSE;
 	}
 
 	return FALSE;
 }
 
-BOOL freerdp_dsp_ffmpeg_encode(FREERDP_DSP_CONTEXT* context, const AUDIO_FORMAT* format,
-                               const BYTE* data, size_t length, wStream* out)
+BOOL freerdp_dsp_ffmpeg_encode(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context,
+                               const AUDIO_FORMAT* WINPR_RESTRICT format,
+                               const BYTE* WINPR_RESTRICT data, size_t length,
+                               wStream* WINPR_RESTRICT out)
 {
-	int rc;
 	AUDIO_FORMAT fmt = { 0 };
 
-	if (!context || !format || !data || !out || !context->encoder)
+	if (!context || !format || !data || !out || !context->common.encoder)
 		return FALSE;
 
 	if (!context || !data || !out)
@@ -799,14 +818,17 @@ BOOL freerdp_dsp_ffmpeg_encode(FREERDP_DSP_CONTEXT* context, const AUDIO_FORMAT*
 			if (inSamples + (int)context->bufferedSamples > context->context->frame_size)
 				inSamples = context->context->frame_size - (int)context->bufferedSamples;
 
-#if LIBAVUTIL_VERSION_INT < AV_VERSION_INT(57, 28, 100)
-			const int channels = context->context->channels;
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 28, 100)
+			const int nrchannels = context->context->ch_layout.nb_channels;
 #else
-			const int channels = context->context->ch_layout.nb_channels;
+			const int nrchannels = context->context->channels;
 #endif
-			rc = av_samples_copy(context->buffered->extended_data,
-			                     context->resampled->extended_data, (int)context->bufferedSamples,
-			                     copied, inSamples, channels, context->context->sample_fmt);
+			const int rc =
+			    av_samples_copy(context->buffered->extended_data, context->resampled->extended_data,
+			                    (int)context->bufferedSamples, copied, inSamples, nrchannels,
+			                    context->context->sample_fmt);
+			if (rc < 0)
+				return FALSE;
 			rest -= inSamples;
 			copied += inSamples;
 			context->bufferedSamples += (UINT32)inSamples;
@@ -825,17 +847,21 @@ BOOL freerdp_dsp_ffmpeg_encode(FREERDP_DSP_CONTEXT* context, const AUDIO_FORMAT*
 	}
 }
 
-BOOL freerdp_dsp_ffmpeg_decode(FREERDP_DSP_CONTEXT* context, const AUDIO_FORMAT* srcFormat,
-                               const BYTE* data, size_t length, wStream* out)
+BOOL freerdp_dsp_ffmpeg_decode(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context,
+                               const AUDIO_FORMAT* WINPR_RESTRICT srcFormat,
+                               const BYTE* WINPR_RESTRICT data, size_t length,
+                               wStream* WINPR_RESTRICT out)
 {
-	if (!context || !srcFormat || !data || !out || context->encoder)
+	if (!context || !srcFormat || !data || !out || context->common.encoder)
 		return FALSE;
 
 #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(58, 133, 100)
 	av_init_packet(context->packet);
 #endif
 	context->packet->data = (uint8_t*)data;
-	context->packet->size = length;
+
+	WINPR_ASSERT(length <= INT_MAX);
+	context->packet->size = (int)length;
 	return ffmpeg_decode(context->context, context->packet, context->frame, context->rcontext,
 	                     context->resampled, out);
 }
