@@ -68,20 +68,22 @@ static const char* openh264_library_names[] = {
 };
 #endif
 
-static void openh264_trace_callback(H264_CONTEXT* h264, int level, const char* message)
+static void openh264_trace_callback(H264_CONTEXT* WINPR_RESTRICT h264, int level,
+                                    const char* WINPR_RESTRICT message)
 {
 	if (h264)
 		WLog_Print(h264->log, WLOG_TRACE, "%d - %s", level, message);
 }
 
-static int openh264_decompress(H264_CONTEXT* h264, const BYTE* pSrcData, UINT32 SrcSize)
+static int openh264_decompress(H264_CONTEXT* WINPR_RESTRICT h264,
+                               const BYTE* WINPR_RESTRICT pSrcData, UINT32 SrcSize)
 {
-	DECODING_STATE state;
+	DECODING_STATE state = dsInvalidArgument;
 	SBufferInfo sBufferInfo = { 0 };
-	SSysMEMBuffer* pSystemBuffer;
-	H264_CONTEXT_OPENH264* sys;
-	UINT32* iStride;
-	BYTE** pYUVData;
+	SSysMEMBuffer* pSystemBuffer = NULL;
+	H264_CONTEXT_OPENH264* sys = NULL;
+	UINT32* iStride = NULL;
+	BYTE** pYUVData = NULL;
 
 	WINPR_ASSERT(h264);
 	WINPR_ASSERT(pSrcData || (SrcSize == 0));
@@ -179,15 +181,16 @@ static int openh264_decompress(H264_CONTEXT* h264, const BYTE* pSrcData, UINT32
 	return 1;
 }
 
-static int openh264_compress(H264_CONTEXT* h264, const BYTE** pYUVData, const UINT32* iStride,
-                             BYTE** ppDstData, UINT32* pDstSize)
+static int openh264_compress(H264_CONTEXT* WINPR_RESTRICT h264,
+                             const BYTE** WINPR_RESTRICT pYUVData,
+                             const UINT32* WINPR_RESTRICT iStride, BYTE** WINPR_RESTRICT ppDstData,
+                             UINT32* WINPR_RESTRICT pDstSize)
 {
-	int i, j;
-	int status;
+	int status = 0;
 	SFrameBSInfo info = { 0 };
 	SSourcePicture pic = { 0 };
 
-	H264_CONTEXT_OPENH264* sys;
+	H264_CONTEXT_OPENH264* sys = NULL;
 
 	WINPR_ASSERT(h264);
 	WINPR_ASSERT(pYUVData);
@@ -225,14 +228,32 @@ static int openh264_compress(H264_CONTEXT* h264, const BYTE** pYUVData, const UI
 			return status;
 		}
 
-		sys->EncParamExt.iUsageType = SCREEN_CONTENT_REAL_TIME;
+		EUsageType usageType;
+
+		switch (h264->UsageType)
+		{
+			case H264_CAMERA_VIDEO_NON_REAL_TIME:
+				usageType = CAMERA_VIDEO_NON_REAL_TIME;
+				break;
+			case H264_CAMERA_VIDEO_REAL_TIME:
+				usageType = CAMERA_VIDEO_REAL_TIME;
+				break;
+			case H264_SCREEN_CONTENT_NON_REAL_TIME:
+				usageType = SCREEN_CONTENT_NON_REAL_TIME;
+				break;
+			case H264_SCREEN_CONTENT_REAL_TIME:
+			default:
+				usageType = SCREEN_CONTENT_REAL_TIME;
+				break;
+		}
+
+		sys->EncParamExt.iUsageType = usageType;
 		sys->EncParamExt.iPicWidth = (int)h264->width;
 		sys->EncParamExt.iPicHeight = (int)h264->height;
 		sys->EncParamExt.fMaxFrameRate = (int)h264->FrameRate;
 		sys->EncParamExt.iMaxBitrate = UNSPECIFIED_BIT_RATE;
 		sys->EncParamExt.bEnableDenoise = 0;
 		sys->EncParamExt.bEnableLongTermReference = 0;
-		sys->EncParamExt.bEnableFrameSkip = 0;
 		sys->EncParamExt.iSpatialLayerNum = 1;
 		sys->EncParamExt.iMultipleThreadIdc = (int)h264->NumberOfThreads;
 		sys->EncParamExt.sSpatialLayers[0].fFrameRate = h264->FrameRate;
@@ -247,11 +268,13 @@ static int openh264_compress(H264_CONTEXT* h264, const BYTE** pYUVData, const UI
 				sys->EncParamExt.iTargetBitrate = (int)h264->BitRate;
 				sys->EncParamExt.sSpatialLayers[0].iSpatialBitrate =
 				    sys->EncParamExt.iTargetBitrate;
+				sys->EncParamExt.bEnableFrameSkip = 1;
 				break;
 
 			case H264_RATECONTROL_CQP:
 				sys->EncParamExt.iRCMode = RC_OFF_MODE;
 				sys->EncParamExt.sSpatialLayers[0].iDLayerQp = (int)h264->QP;
+				sys->EncParamExt.bEnableFrameSkip = 0;
 				break;
 		}
 
@@ -374,9 +397,9 @@ static int openh264_compress(H264_CONTEXT* h264, const BYTE** pYUVData, const UI
 	*ppDstData = info.sLayerInfo[0].pBsBuf;
 	*pDstSize = 0;
 
-	for (i = 0; i < info.iLayerNum; i++)
+	for (int i = 0; i < info.iLayerNum; i++)
 	{
-		for (j = 0; j < info.sLayerInfo[i].iNalCount; j++)
+		for (int j = 0; j < info.sLayerInfo[i].iNalCount; j++)
 		{
 			*pDstSize += info.sLayerInfo[i].pNalLengthInByte[j];
 		}
@@ -387,8 +410,7 @@ static int openh264_compress(H264_CONTEXT* h264, const BYTE** pYUVData, const UI
 
 static void openh264_uninit(H264_CONTEXT* h264)
 {
-	UINT32 x;
-	H264_CONTEXT_OPENH264* sysContexts;
+	H264_CONTEXT_OPENH264* sysContexts = NULL;
 
 	WINPR_ASSERT(h264);
 
@@ -396,7 +418,7 @@ static void openh264_uninit(H264_CONTEXT* h264)
 
 	if (sysContexts)
 	{
-		for (x = 0; x < h264->numSystemData; x++)
+		for (UINT32 x = 0; x < h264->numSystemData; x++)
 		{
 			H264_CONTEXT_OPENH264* sys = &sysContexts[x];
 
@@ -489,11 +511,9 @@ static BOOL openh264_init(H264_CONTEXT* h264)
 {
 #if defined(WITH_OPENH264_LOADING)
 	BOOL success = FALSE;
-	size_t i;
 #endif
-	UINT32 x;
-	long status;
-	H264_CONTEXT_OPENH264* sysContexts;
+	long status = 0;
+	H264_CONTEXT_OPENH264* sysContexts = NULL;
 	static int traceLevel = WELS_LOG_DEBUG;
 #if (OPENH264_MAJOR == 1) && (OPENH264_MINOR <= 5)
 	static EVideoFormatType videoFormat = videoFormatI420;
@@ -512,7 +532,7 @@ static BOOL openh264_init(H264_CONTEXT* h264)
 	h264->pSystemData = (void*)sysContexts;
 #if defined(WITH_OPENH264_LOADING)
 
-	for (i = 0; i < ARRAYSIZE(openh264_library_names); i++)
+	for (size_t i = 0; i < ARRAYSIZE(openh264_library_names); i++)
 	{
 		const char* current = openh264_library_names[i];
 		success = openh264_load_functionpointers(h264, current);
@@ -532,7 +552,7 @@ static BOOL openh264_init(H264_CONTEXT* h264)
 	sysContexts->WelsDestroySVCEncoder = WelsDestroySVCEncoder;
 #endif
 
-	for (x = 0; x < h264->numSystemData; x++)
+	for (UINT32 x = 0; x < h264->numSystemData; x++)
 	{
 		SDecodingParam sDecParam = { 0 };
 		H264_CONTEXT_OPENH264* sys = &sysContexts[x];
