@@ -67,9 +67,10 @@ struct S_YUV_CONTEXT
 	YUV_COMBINE_WORK_PARAM* work_combined_params;
 };
 
-static INLINE BOOL avc420_yuv_to_rgb(const BYTE* pYUVData[3], const UINT32 iStride[3],
-                                     const RECTANGLE_16* rect, UINT32 nDstStep, BYTE* pDstData,
-                                     DWORD DstFormat)
+static INLINE BOOL avc420_yuv_to_rgb(const BYTE* WINPR_RESTRICT pYUVData[3],
+                                     const UINT32 iStride[3],
+                                     const RECTANGLE_16* WINPR_RESTRICT rect, UINT32 nDstStep,
+                                     BYTE* WINPR_RESTRICT pDstData, DWORD DstFormat)
 {
 	primitives_t* prims = primitives_get();
 	prim_size_t roi;
@@ -99,9 +100,10 @@ static INLINE BOOL avc420_yuv_to_rgb(const BYTE* pYUVData[3], const UINT32 iStri
 	return TRUE;
 }
 
-static INLINE BOOL avc444_yuv_to_rgb(const BYTE* pYUVData[3], const UINT32 iStride[3],
-                                     const RECTANGLE_16* rect, UINT32 nDstStep, BYTE* pDstData,
-                                     DWORD DstFormat)
+static INLINE BOOL avc444_yuv_to_rgb(const BYTE* WINPR_RESTRICT pYUVData[3],
+                                     const UINT32 iStride[3],
+                                     const RECTANGLE_16* WINPR_RESTRICT rect, UINT32 nDstStep,
+                                     BYTE* WINPR_RESTRICT pDstData, DWORD DstFormat)
 {
 	primitives_t* prims = primitives_get();
 	prim_size_t roi;
@@ -157,7 +159,7 @@ static void CALLBACK yuv444_process_work_callback(PTP_CALLBACK_INSTANCE instance
 		WLog_WARN(TAG, "avc444_yuv_to_rgb failed");
 }
 
-BOOL yuv_context_reset(YUV_CONTEXT* context, UINT32 width, UINT32 height)
+BOOL yuv_context_reset(YUV_CONTEXT* WINPR_RESTRICT context, UINT32 width, UINT32 height)
 {
 	BOOL rc = FALSE;
 	WINPR_ASSERT(context);
@@ -251,7 +253,10 @@ YUV_CONTEXT* yuv_context_new(BOOL encoder, UINT32 ThreadingFlags)
 	return ret;
 
 error_threadpool:
+	WINPR_PRAGMA_DIAG_PUSH
+	WINPR_PRAGMA_DIAG_IGNORED_MISMATCHED_DEALLOC
 	yuv_context_free(ret);
+	WINPR_PRAGMA_DIAG_POP
 	return NULL;
 }
 
@@ -272,11 +277,11 @@ void yuv_context_free(YUV_CONTEXT* context)
 	winpr_aligned_free(context);
 }
 
-static INLINE YUV_PROCESS_WORK_PARAM pool_decode_param(const RECTANGLE_16* rect,
-                                                       YUV_CONTEXT* context,
-                                                       const BYTE* pYUVData[3],
+static INLINE YUV_PROCESS_WORK_PARAM pool_decode_param(const RECTANGLE_16* WINPR_RESTRICT rect,
+                                                       YUV_CONTEXT* WINPR_RESTRICT context,
+                                                       const BYTE* WINPR_RESTRICT pYUVData[3],
                                                        const UINT32 iStride[3], UINT32 DstFormat,
-                                                       BYTE* dest, UINT32 nDstStep)
+                                                       BYTE* WINPR_RESTRICT dest, UINT32 nDstStep)
 {
 	YUV_PROCESS_WORK_PARAM current = { 0 };
 
@@ -300,8 +305,8 @@ static INLINE YUV_PROCESS_WORK_PARAM pool_decode_param(const RECTANGLE_16* rect,
 	return current;
 }
 
-static BOOL submit_object(PTP_WORK* work_object, PTP_WORK_CALLBACK cb, const void* param,
-                          YUV_CONTEXT* context)
+static BOOL submit_object(PTP_WORK* WINPR_RESTRICT work_object, PTP_WORK_CALLBACK cb,
+                          const void* WINPR_RESTRICT param, YUV_CONTEXT* WINPR_RESTRICT context)
 {
 	union
 	{
@@ -329,11 +334,9 @@ static BOOL submit_object(PTP_WORK* work_object, PTP_WORK_CALLBACK cb, const voi
 
 static void free_objects(PTP_WORK* work_objects, UINT32 waitCount)
 {
-	UINT32 i;
-
 	WINPR_ASSERT(work_objects || (waitCount == 0));
 
-	for (i = 0; i < waitCount; i++)
+	for (UINT32 i = 0; i < waitCount; i++)
 	{
 		PTP_WORK cur = work_objects[i];
 		work_objects[i] = NULL;
@@ -346,13 +349,12 @@ static void free_objects(PTP_WORK* work_objects, UINT32 waitCount)
 	}
 }
 
-static BOOL intersects(UINT32 pos, const RECTANGLE_16* regionRects, UINT32 numRegionRects)
+static BOOL intersects(UINT32 pos, const RECTANGLE_16* WINPR_RESTRICT regionRects,
+                       UINT32 numRegionRects)
 {
-	UINT32 x;
-
 	WINPR_ASSERT(regionRects || (numRegionRects == 0));
 
-	for (x = pos + 1; x < numRegionRects; x++)
+	for (UINT32 x = pos + 1; x < numRegionRects; x++)
 	{
 		const RECTANGLE_16* what = &regionRects[pos];
 		const RECTANGLE_16* rect = &regionRects[x];
@@ -367,7 +369,8 @@ static BOOL intersects(UINT32 pos, const RECTANGLE_16* regionRects, UINT32 numRe
 	return FALSE;
 }
 
-static RECTANGLE_16 clamp(YUV_CONTEXT* context, const RECTANGLE_16* rect, UINT32 srcHeight)
+static RECTANGLE_16 clamp(YUV_CONTEXT* WINPR_RESTRICT context,
+                          const RECTANGLE_16* WINPR_RESTRICT rect, UINT32 srcHeight)
 {
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(rect);
@@ -381,12 +384,13 @@ static RECTANGLE_16 clamp(YUV_CONTEXT* context, const RECTANGLE_16* rect, UINT32
 	return c;
 }
 
-static BOOL pool_decode(YUV_CONTEXT* context, PTP_WORK_CALLBACK cb, const BYTE* pYUVData[3],
-                        const UINT32 iStride[3], UINT32 yuvHeight, UINT32 DstFormat, BYTE* dest,
-                        UINT32 nDstStep, const RECTANGLE_16* regionRects, UINT32 numRegionRects)
+static BOOL pool_decode(YUV_CONTEXT* WINPR_RESTRICT context, PTP_WORK_CALLBACK cb,
+                        const BYTE* WINPR_RESTRICT pYUVData[3], const UINT32 iStride[3],
+                        UINT32 yuvHeight, UINT32 DstFormat, BYTE* WINPR_RESTRICT dest,
+                        UINT32 nDstStep, const RECTANGLE_16* WINPR_RESTRICT regionRects,
+                        UINT32 numRegionRects)
 {
 	BOOL rc = FALSE;
-	UINT32 x;
 	UINT32 waitCount = 0;
 	primitives_t* prims = primitives_get();
 
@@ -416,7 +420,7 @@ static BOOL pool_decode(YUV_CONTEXT* context, PTP_WORK_CALLBACK cb, const BYTE*
 	}
 
 	/* case where we use threads */
-	for (x = 0; x < numRegionRects; x++)
+	for (UINT32 x = 0; x < numRegionRects; x++)
 	{
 		RECTANGLE_16 r = clamp(context, &regionRects[x], yuvHeight);
 
@@ -461,7 +465,8 @@ fail:
 	return rc;
 }
 
-static INLINE BOOL check_rect(const YUV_CONTEXT* yuv, const RECTANGLE_16* rect, UINT32 nDstWidth,
+static INLINE BOOL check_rect(const YUV_CONTEXT* WINPR_RESTRICT yuv,
+                              const RECTANGLE_16* WINPR_RESTRICT rect, UINT32 nDstWidth,
                               UINT32 nDstHeight)
 {
 	WINPR_ASSERT(yuv);
@@ -513,9 +518,10 @@ static void CALLBACK yuv444_combine_work_callback(PTP_CALLBACK_INSTANCE instance
 		WLog_WARN(TAG, "YUV420CombineToYUV444 failed");
 }
 
-static INLINE YUV_COMBINE_WORK_PARAM pool_decode_rect_param(
-    const RECTANGLE_16* rect, YUV_CONTEXT* context, BYTE type, const BYTE* pYUVData[3],
-    const UINT32 iStride[3], BYTE* pYUVDstData[3], const UINT32 iDstStride[3])
+static INLINE YUV_COMBINE_WORK_PARAM
+pool_decode_rect_param(const RECTANGLE_16* WINPR_RESTRICT rect, YUV_CONTEXT* WINPR_RESTRICT context,
+                       BYTE type, const BYTE* WINPR_RESTRICT pYUVData[3], const UINT32 iStride[3],
+                       BYTE* WINPR_RESTRICT pYUVDstData[3], const UINT32 iDstStride[3])
 {
 	YUV_COMBINE_WORK_PARAM current = { 0 };
 
@@ -544,13 +550,12 @@ static INLINE YUV_COMBINE_WORK_PARAM pool_decode_rect_param(
 	return current;
 }
 
-static BOOL pool_decode_rect(YUV_CONTEXT* context, BYTE type, const BYTE* pYUVData[3],
-                             const UINT32 iStride[3], BYTE* pYUVDstData[3],
-                             const UINT32 iDstStride[3], const RECTANGLE_16* regionRects,
-                             UINT32 numRegionRects)
+static BOOL pool_decode_rect(YUV_CONTEXT* WINPR_RESTRICT context, BYTE type,
+                             const BYTE* WINPR_RESTRICT pYUVData[3], const UINT32 iStride[3],
+                             BYTE* WINPR_RESTRICT pYUVDstData[3], const UINT32 iDstStride[3],
+                             const RECTANGLE_16* WINPR_RESTRICT regionRects, UINT32 numRegionRects)
 {
 	BOOL rc = FALSE;
-	UINT32 y;
 	UINT32 waitCount = 0;
 	PTP_WORK_CALLBACK cb = yuv444_combine_work_callback;
 	primitives_t* prims = primitives_get();
@@ -564,7 +569,7 @@ static BOOL pool_decode_rect(YUV_CONTEXT* context, BYTE type, const BYTE* pYUVDa
 
 	if (!context->useThreads || (primitives_flags(prims) & PRIM_FLAGS_HAVE_EXTGPU))
 	{
-		for (y = 0; y < numRegionRects; y++)
+		for (UINT32 y = 0; y < numRegionRects; y++)
 		{
 			YUV_COMBINE_WORK_PARAM current = pool_decode_rect_param(
 			    &regionRects[y], context, type, pYUVData, iStride, pYUVDstData, iDstStride);
@@ -576,7 +581,7 @@ static BOOL pool_decode_rect(YUV_CONTEXT* context, BYTE type, const BYTE* pYUVDa
 	/* case where we use threads */
 	for (waitCount = 0; waitCount < numRegionRects; waitCount++)
 	{
-		YUV_COMBINE_WORK_PARAM* current;
+		YUV_COMBINE_WORK_PARAM* current = NULL;
 
 		if (context->work_object_count <= waitCount)
 		{
@@ -600,10 +605,12 @@ fail:
 	return rc;
 }
 
-BOOL yuv444_context_decode(YUV_CONTEXT* context, BYTE type, const BYTE* pYUVData[3],
-                           const UINT32 iStride[3], UINT32 srcYuvHeight, BYTE* pYUVDstData[3],
-                           const UINT32 iDstStride[3], DWORD DstFormat, BYTE* dest, UINT32 nDstStep,
-                           const RECTANGLE_16* regionRects, UINT32 numRegionRects)
+BOOL yuv444_context_decode(YUV_CONTEXT* WINPR_RESTRICT context, BYTE type,
+                           const BYTE* WINPR_RESTRICT pYUVData[3], const UINT32 iStride[3],
+                           UINT32 srcYuvHeight, BYTE* WINPR_RESTRICT pYUVDstData[3],
+                           const UINT32 iDstStride[3], DWORD DstFormat, BYTE* WINPR_RESTRICT dest,
+                           UINT32 nDstStep, const RECTANGLE_16* WINPR_RESTRICT regionRects,
+                           UINT32 numRegionRects)
 {
 	const BYTE* pYUVCDstData[3];
 
@@ -631,9 +638,11 @@ BOOL yuv444_context_decode(YUV_CONTEXT* context, BYTE type, const BYTE* pYUVData
 	                   srcYuvHeight, DstFormat, dest, nDstStep, regionRects, numRegionRects);
 }
 
-BOOL yuv420_context_decode(YUV_CONTEXT* context, const BYTE* pYUVData[3], const UINT32 iStride[3],
-                           UINT32 yuvHeight, DWORD DstFormat, BYTE* dest, UINT32 nDstStep,
-                           const RECTANGLE_16* regionRects, UINT32 numRegionRects)
+BOOL yuv420_context_decode(YUV_CONTEXT* WINPR_RESTRICT context,
+                           const BYTE* WINPR_RESTRICT pYUVData[3], const UINT32 iStride[3],
+                           UINT32 yuvHeight, DWORD DstFormat, BYTE* WINPR_RESTRICT dest,
+                           UINT32 nDstStep, const RECTANGLE_16* WINPR_RESTRICT regionRects,
+                           UINT32 numRegionRects)
 {
 	return pool_decode(context, yuv420_process_work_callback, pYUVData, iStride, yuvHeight,
 	                   DstFormat, dest, nDstStep, regionRects, numRegionRects);
@@ -646,7 +655,7 @@ static void CALLBACK yuv420_encode_work_callback(PTP_CALLBACK_INSTANCE instance,
 	YUV_ENCODE_WORK_PARAM* param = (YUV_ENCODE_WORK_PARAM*)context;
 	primitives_t* prims = primitives_get();
 	BYTE* pYUVData[3];
-	const BYTE* src;
+	const BYTE* src = NULL;
 
 	WINPR_UNUSED(instance);
 	WINPR_UNUSED(work);
@@ -677,7 +686,7 @@ static void CALLBACK yuv444v1_encode_work_callback(PTP_CALLBACK_INSTANCE instanc
 	primitives_t* prims = primitives_get();
 	BYTE* pYUVLumaData[3];
 	BYTE* pYUVChromaData[3];
-	const BYTE* src;
+	const BYTE* src = NULL;
 
 	WINPR_UNUSED(instance);
 	WINPR_UNUSED(work);
@@ -714,7 +723,7 @@ static void CALLBACK yuv444v2_encode_work_callback(PTP_CALLBACK_INSTANCE instanc
 	primitives_t* prims = primitives_get();
 	BYTE* pYUVLumaData[3];
 	BYTE* pYUVChromaData[3];
-	const BYTE* src;
+	const BYTE* src = NULL;
 
 	WINPR_UNUSED(instance);
 	WINPR_UNUSED(work);
@@ -744,10 +753,10 @@ static void CALLBACK yuv444v2_encode_work_callback(PTP_CALLBACK_INSTANCE instanc
 	}
 }
 
-static INLINE YUV_ENCODE_WORK_PARAM pool_encode_fill(const RECTANGLE_16* rect, YUV_CONTEXT* context,
-                                                     const BYTE* pSrcData, UINT32 nSrcStep,
-                                                     UINT32 SrcFormat, const UINT32 iStride[],
-                                                     BYTE* pYUVLumaData[], BYTE* pYUVChromaData[])
+static INLINE YUV_ENCODE_WORK_PARAM pool_encode_fill(
+    const RECTANGLE_16* WINPR_RESTRICT rect, YUV_CONTEXT* WINPR_RESTRICT context,
+    const BYTE* WINPR_RESTRICT pSrcData, UINT32 nSrcStep, UINT32 SrcFormat, const UINT32 iStride[],
+    BYTE* WINPR_RESTRICT pYUVLumaData[], BYTE* WINPR_RESTRICT pYUVChromaData[])
 {
 	YUV_ENCODE_WORK_PARAM current = { 0 };
 
@@ -779,14 +788,14 @@ static INLINE YUV_ENCODE_WORK_PARAM pool_encode_fill(const RECTANGLE_16* rect, Y
 	return current;
 }
 
-static BOOL pool_encode(YUV_CONTEXT* context, PTP_WORK_CALLBACK cb, const BYTE* pSrcData,
-                        UINT32 nSrcStep, UINT32 SrcFormat, const UINT32 iStride[],
-                        BYTE* pYUVLumaData[], BYTE* pYUVChromaData[],
-                        const RECTANGLE_16* regionRects, UINT32 numRegionRects)
+static BOOL pool_encode(YUV_CONTEXT* WINPR_RESTRICT context, PTP_WORK_CALLBACK cb,
+                        const BYTE* WINPR_RESTRICT pSrcData, UINT32 nSrcStep, UINT32 SrcFormat,
+                        const UINT32 iStride[], BYTE* WINPR_RESTRICT pYUVLumaData[],
+                        BYTE* WINPR_RESTRICT pYUVChromaData[],
+                        const RECTANGLE_16* WINPR_RESTRICT regionRects, UINT32 numRegionRects)
 {
 	BOOL rc = FALSE;
 	primitives_t* prims = primitives_get();
-	UINT32 x, y;
 	UINT32 waitCount = 0;
 
 	WINPR_ASSERT(context);
@@ -804,7 +813,7 @@ static BOOL pool_encode(YUV_CONTEXT* context, PTP_WORK_CALLBACK cb, const BYTE*
 
 	if (!context->useThreads || (primitives_flags(prims) & PRIM_FLAGS_HAVE_EXTGPU))
 	{
-		for (x = 0; x < numRegionRects; x++)
+		for (UINT32 x = 0; x < numRegionRects; x++)
 		{
 			YUV_ENCODE_WORK_PARAM current =
 			    pool_encode_fill(&regionRects[x], context, pSrcData, nSrcStep, SrcFormat, iStride,
@@ -815,7 +824,7 @@ static BOOL pool_encode(YUV_CONTEXT* context, PTP_WORK_CALLBACK cb, const BYTE*
 	}
 
 	/* case where we use threads */
-	for (x = 0; x < numRegionRects; x++)
+	for (UINT32 x = 0; x < numRegionRects; x++)
 	{
 		const RECTANGLE_16* rect = &regionRects[x];
 		const UINT32 height = rect->bottom - rect->top;
@@ -824,16 +833,16 @@ static BOOL pool_encode(YUV_CONTEXT* context, PTP_WORK_CALLBACK cb, const BYTE*
 		waitCount += steps;
 	}
 
-	for (x = 0; x < numRegionRects; x++)
+	for (UINT32 x = 0; x < numRegionRects; x++)
 	{
 		const RECTANGLE_16* rect = &regionRects[x];
 		const UINT32 height = rect->bottom - rect->top;
 		const UINT32 steps = (height + context->heightStep / 2) / context->heightStep;
 
-		for (y = 0; y < steps; y++)
+		for (UINT32 y = 0; y < steps; y++)
 		{
 			RECTANGLE_16 r = *rect;
-			YUV_ENCODE_WORK_PARAM* current;
+			YUV_ENCODE_WORK_PARAM* current = NULL;
 
 			if (context->work_object_count <= waitCount)
 			{
@@ -860,9 +869,10 @@ fail:
 	return rc;
 }
 
-BOOL yuv420_context_encode(YUV_CONTEXT* context, const BYTE* pSrcData, UINT32 nSrcStep,
-                           UINT32 SrcFormat, const UINT32 iStride[3], BYTE* pYUVData[3],
-                           const RECTANGLE_16* regionRects, UINT32 numRegionRects)
+BOOL yuv420_context_encode(YUV_CONTEXT* WINPR_RESTRICT context, const BYTE* WINPR_RESTRICT pSrcData,
+                           UINT32 nSrcStep, UINT32 SrcFormat, const UINT32 iStride[3],
+                           BYTE* WINPR_RESTRICT pYUVData[3],
+                           const RECTANGLE_16* WINPR_RESTRICT regionRects, UINT32 numRegionRects)
 {
 	if (!context || !pSrcData || !iStride || !pYUVData || !regionRects)
 		return FALSE;
@@ -871,12 +881,13 @@ BOOL yuv420_context_encode(YUV_CONTEXT* context, const BYTE* pSrcData, UINT32 nS
 	                   pYUVData, NULL, regionRects, numRegionRects);
 }
 
-BOOL yuv444_context_encode(YUV_CONTEXT* context, BYTE version, const BYTE* pSrcData,
-                           UINT32 nSrcStep, UINT32 SrcFormat, const UINT32 iStride[3],
-                           BYTE* pYUVLumaData[3], BYTE* pYUVChromaData[3],
-                           const RECTANGLE_16* regionRects, UINT32 numRegionRects)
+BOOL yuv444_context_encode(YUV_CONTEXT* WINPR_RESTRICT context, BYTE version,
+                           const BYTE* WINPR_RESTRICT pSrcData, UINT32 nSrcStep, UINT32 SrcFormat,
+                           const UINT32 iStride[3], BYTE* WINPR_RESTRICT pYUVLumaData[3],
+                           BYTE* WINPR_RESTRICT pYUVChromaData[3],
+                           const RECTANGLE_16* WINPR_RESTRICT regionRects, UINT32 numRegionRects)
 {
-	PTP_WORK_CALLBACK cb;
+	PTP_WORK_CALLBACK cb = NULL;
 	switch (version)
 	{
 		case 1:
