@@ -1456,7 +1456,10 @@ freerdp_peer* freerdp_peer_new(int sockfd)
 	option_len = sizeof(option_value);
 
 	if (sockfd >= 0)
-		setsockopt(sockfd, IPPROTO_TCP, TCP_NODELAY, (void*)&option_value, option_len);
+	{
+		if (setsockopt(sockfd, IPPROTO_TCP, TCP_NODELAY, (void*)&option_value, option_len) < 0)
+			goto fail;
+	}
 
 	if (client)
 	{
@@ -1487,6 +1490,10 @@ freerdp_peer* freerdp_peer_new(int sockfd)
 	}
 
 	return client;
+
+fail:
+	freerdp_peer_free(client);
+	return NULL;
 }
 
 void freerdp_peer_free(freerdp_peer* client)
@@ -1495,7 +1502,8 @@ void freerdp_peer_free(freerdp_peer* client)
 		return;
 
 	sspi_FreeAuthIdentity(&client->identity);
-	closesocket((SOCKET)client->sockfd);
+	if (client->sockfd >= 0)
+		closesocket((SOCKET)client->sockfd);
 	free(client);
 }
 
@@ -1511,6 +1519,7 @@ static BOOL freerdp_peer_transport_setup(freerdp_peer* client)
 
 	if (!transport_attach(rdp->transport, client->sockfd))
 		return FALSE;
+	client->sockfd = -1;
 
 	if (!transport_set_recv_callbacks(rdp->transport, peer_recv_callback, client))
 		return FALSE;
@@ -1530,6 +1539,7 @@ BOOL freerdp_peer_context_new_ex(freerdp_peer* client, const rdpSettings* settin
 	if (!client)
 		return FALSE;
 
+	WINPR_ASSERT(client->ContextSize >= sizeof(rdpContext));
 	if (!(context = (rdpContext*)calloc(1, client->ContextSize)))
 		goto fail;
 
