@@ -53,6 +53,19 @@ typedef enum
 	RDSTLS_STATE_FINAL,
 } RDSTLS_STATE;
 
+typedef enum
+{
+
+	RDSTLS_RESULT_SUCCESS = 0x00000000,
+	RDSTLS_RESULT_ACCESS_DENIED = 0x00000005,
+	RDSTLS_RESULT_LOGON_FAILURE = 0x0000052e,
+	RDSTLS_RESULT_INVALID_LOGON_HOURS = 0x00000530,
+	RDSTLS_RESULT_PASSWORD_EXPIRED = 0x00000532,
+	RDSTLS_RESULT_ACCOUNT_DISABLED = 0x00000533,
+	RDSTLS_RESULT_PASSWORD_MUST_CHANGE = 0x00000773,
+	RDSTLS_RESULT_ACCOUNT_LOCKED_OUT = 0x00000775
+} RDSTLS_RESULT_CODE;
+
 struct rdp_rdstls
 {
 	BOOL server;
@@ -60,10 +73,34 @@ struct rdp_rdstls
 	rdpContext* context;
 	rdpTransport* transport;
 
-	UINT32 resultCode;
+	RDSTLS_RESULT_CODE resultCode;
 	wLog* log;
 };
 
+static const char* rdstls_result_code_str(UINT32 resultCode)
+{
+	switch (resultCode)
+	{
+		case RDSTLS_RESULT_SUCCESS:
+			return "RDSTLS_RESULT_SUCCESS";
+		case RDSTLS_RESULT_ACCESS_DENIED:
+			return "RDSTLS_RESULT_ACCESS_DENIED";
+		case RDSTLS_RESULT_LOGON_FAILURE:
+			return "RDSTLS_RESULT_LOGON_FAILURE";
+		case RDSTLS_RESULT_INVALID_LOGON_HOURS:
+			return "RDSTLS_RESULT_INVALID_LOGON_HOURS";
+		case RDSTLS_RESULT_PASSWORD_EXPIRED:
+			return "RDSTLS_RESULT_PASSWORD_EXPIRED";
+		case RDSTLS_RESULT_ACCOUNT_DISABLED:
+			return "RDSTLS_RESULT_ACCOUNT_DISABLED";
+		case RDSTLS_RESULT_PASSWORD_MUST_CHANGE:
+			return "RDSTLS_RESULT_PASSWORD_MUST_CHANGE";
+		case RDSTLS_RESULT_ACCOUNT_LOCKED_OUT:
+			return "RDSTLS_RESULT_ACCOUNT_LOCKED_OUT";
+		default:
+			return "RDSTLS_RESULT_UNKNOWN";
+	}
+}
 /**
  * Create new RDSTLS state machine.
  *
@@ -287,8 +324,8 @@ static BOOL rdstls_write_authentication_response(rdpRdstls* rdstls, wStream* s)
 
 static BOOL rdstls_process_capabilities(rdpRdstls* rdstls, wStream* s)
 {
-	UINT16 dataType;
-	UINT16 supportedVersions;
+	UINT16 dataType = 0;
+	UINT16 supportedVersions = 0;
 
 	if (!Stream_CheckAndLogRequiredLengthWLog(rdstls->log, s, 4))
 		return FALSE;
@@ -451,21 +488,21 @@ static BOOL rdstls_process_authentication_request_with_password(rdpRdstls* rdstl
 	serverDomain = freerdp_settings_get_string(settings, FreeRDP_Domain);
 	serverPassword = freerdp_settings_get_string(settings, FreeRDP_Password);
 
-	rdstls->resultCode = ERROR_SUCCESS;
+	rdstls->resultCode = RDSTLS_RESULT_SUCCESS;
 
 	if (!rdstls_cmp_data(rdstls->log, "RedirectionGuid", serverRedirectionGuid,
 	                     serverRedirectionGuidLength, clientRedirectionGuid,
 	                     clientRedirectionGuidLength))
-		rdstls->resultCode = ERROR_LOGON_FAILURE;
+		rdstls->resultCode = RDSTLS_RESULT_ACCESS_DENIED;
 
 	if (!rdstls_cmp_str(rdstls->log, "UserName", serverUsername, clientUsername))
-		rdstls->resultCode = ERROR_LOGON_FAILURE;
+		rdstls->resultCode = RDSTLS_RESULT_LOGON_FAILURE;
 
 	if (!rdstls_cmp_str(rdstls->log, "Domain", serverDomain, clientDomain))
-		rdstls->resultCode = ERROR_LOGON_FAILURE;
+		rdstls->resultCode = RDSTLS_RESULT_LOGON_FAILURE;
 
 	if (!rdstls_cmp_str(rdstls->log, "Password", serverPassword, clientPassword))
-		rdstls->resultCode = ERROR_LOGON_FAILURE;
+		rdstls->resultCode = RDSTLS_RESULT_LOGON_FAILURE;
 
 	rc = TRUE;
 fail:
@@ -480,7 +517,7 @@ static BOOL rdstls_process_authentication_request_with_cookie(rdpRdstls* rdstls,
 
 static BOOL rdstls_process_authentication_request(rdpRdstls* rdstls, wStream* s)
 {
-	UINT16 dataType;
+	UINT16 dataType = 0;
 
 	if (!Stream_CheckAndLogRequiredLengthWLog(rdstls->log, s, 2))
 		return FALSE;
@@ -509,8 +546,8 @@ static BOOL rdstls_process_authentication_request(rdpRdstls* rdstls, wStream* s)
 
 static BOOL rdstls_process_authentication_response(rdpRdstls* rdstls, wStream* s)
 {
-	UINT16 dataType;
-	UINT32 resultCode;
+	UINT16 dataType = 0;
+	UINT32 resultCode = 0;
 
 	if (!Stream_CheckAndLogRequiredLengthWLog(rdstls->log, s, 6))
 		return FALSE;
@@ -525,11 +562,41 @@ static BOOL rdstls_process_authentication_response(rdpRdstls* rdstls, wStream* s
 	}
 
 	Stream_Read_UINT32(s, resultCode);
-	if (resultCode != ERROR_SUCCESS)
+	if (resultCode != RDSTLS_RESULT_SUCCESS)
 	{
-		WLog_Print(rdstls->log, WLOG_ERROR, "resultCode: %s [0x%08" PRIX32 "] %s",
-		           freerdp_get_last_error_name(resultCode), resultCode,
-		           freerdp_get_last_error_string(resultCode));
+		WLog_Print(rdstls->log, WLOG_ERROR, "resultCode: %s [0x%08" PRIX32 "]",
+		           rdstls_result_code_str(resultCode), resultCode);
+
+		UINT32 error;
+		switch (resultCode)
+		{
+			case RDSTLS_RESULT_ACCESS_DENIED:
+				error = FREERDP_ERROR_CONNECT_ACCESS_DENIED;
+				break;
+			case RDSTLS_RESULT_ACCOUNT_DISABLED:
+				error = FREERDP_ERROR_CONNECT_ACCOUNT_DISABLED;
+				break;
+			case RDSTLS_RESULT_ACCOUNT_LOCKED_OUT:
+				error = FREERDP_ERROR_CONNECT_ACCOUNT_LOCKED_OUT;
+				break;
+			case RDSTLS_RESULT_LOGON_FAILURE:
+				error = FREERDP_ERROR_CONNECT_LOGON_FAILURE;
+				break;
+			case RDSTLS_RESULT_INVALID_LOGON_HOURS:
+				error = FREERDP_ERROR_CONNECT_ACCOUNT_RESTRICTION;
+				break;
+			case RDSTLS_RESULT_PASSWORD_EXPIRED:
+				error = FREERDP_ERROR_CONNECT_PASSWORD_EXPIRED;
+				break;
+			case RDSTLS_RESULT_PASSWORD_MUST_CHANGE:
+				error = FREERDP_ERROR_CONNECT_PASSWORD_MUST_CHANGE;
+				break;
+			default:
+				error = ERROR_INVALID_PARAMETER;
+				break;
+		}
+
+		freerdp_set_last_error_if_not(rdstls->context, error);
 		return FALSE;
 	}
 
@@ -596,8 +663,8 @@ static BOOL rdstls_send(rdpTransport* transport, wStream* s, void* extra)
 
 static int rdstls_recv(rdpTransport* transport, wStream* s, void* extra)
 {
-	UINT16 version;
-	UINT16 pduType;
+	UINT16 version = 0;
+	UINT16 pduType = 0;
 	rdpRdstls* rdstls = (rdpRdstls*)extra;
 
 	WINPR_ASSERT(transport);
@@ -683,7 +750,7 @@ fail:
 static BOOL rdstls_recv_authentication_request(rdpRdstls* rdstls)
 {
 	BOOL rc = FALSE;
-	int status;
+	int status = 0;
 	wStream* s = NULL;
 
 	if (!rdstls_check_state_requirements(rdstls, RDSTLS_STATE_AUTH_REQ))
@@ -733,7 +800,7 @@ fail:
 static BOOL rdstls_recv_capabilities(rdpRdstls* rdstls)
 {
 	BOOL rc = FALSE;
-	int status;
+	int status = 0;
 	wStream* s = NULL;
 
 	if (!rdstls_check_state_requirements(rdstls, RDSTLS_STATE_CAPABILITIES))
@@ -783,7 +850,7 @@ fail:
 static BOOL rdstls_recv_authentication_response(rdpRdstls* rdstls)
 {
 	BOOL rc = FALSE;
-	int status;
+	int status = 0;
 	wStream* s = NULL;
 
 	WINPR_ASSERT(rdstls);
@@ -825,7 +892,7 @@ static int rdstls_server_authenticate(rdpRdstls* rdstls)
 	if (!rdstls_send_authentication_response(rdstls))
 		return -1;
 
-	if (rdstls->resultCode != 0)
+	if (rdstls->resultCode != RDSTLS_RESULT_SUCCESS)
 		return -1;
 
 	return 1;
@@ -871,7 +938,7 @@ static SSIZE_T rdstls_parse_pdu_data_type(wLog* log, UINT16 dataType, wStream* s
 	{
 		case RDSTLS_DATA_PASSWORD_CREDS:
 		{
-			UINT16 redirGuidLength;
+			UINT16 redirGuidLength = 0;
 			if (Stream_GetRemainingLength(s) < 2)
 				return 0;
 			Stream_Read_UINT16(s, redirGuidLength);
@@ -880,7 +947,7 @@ static SSIZE_T rdstls_parse_pdu_data_type(wLog* log, UINT16 dataType, wStream* s
 				return 0;
 			Stream_Seek(s, redirGuidLength);
 
-			UINT16 usernameLength;
+			UINT16 usernameLength = 0;
 			if (Stream_GetRemainingLength(s) < 2)
 				return 0;
 			Stream_Read_UINT16(s, usernameLength);
@@ -889,7 +956,7 @@ static SSIZE_T rdstls_parse_pdu_data_type(wLog* log, UINT16 dataType, wStream* s
 				return 0;
 			Stream_Seek(s, usernameLength);
 
-			UINT16 domainLength;
+			UINT16 domainLength = 0;
 			if (Stream_GetRemainingLength(s) < 2)
 				return 0;
 			Stream_Read_UINT16(s, domainLength);
@@ -898,7 +965,7 @@ static SSIZE_T rdstls_parse_pdu_data_type(wLog* log, UINT16 dataType, wStream* s
 				return 0;
 			Stream_Seek(s, domainLength);
 
-			UINT16 passwordLength;
+			UINT16 passwordLength = 0;
 			if (Stream_GetRemainingLength(s) < 2)
 				return 0;
 			Stream_Read_UINT16(s, passwordLength);
@@ -911,7 +978,7 @@ static SSIZE_T rdstls_parse_pdu_data_type(wLog* log, UINT16 dataType, wStream* s
 				return 0;
 			Stream_Seek(s, 4);
 
-			UINT16 cookieLength;
+			UINT16 cookieLength = 0;
 			if (Stream_GetRemainingLength(s) < 2)
 				return 0;
 			Stream_Read_UINT16(s, cookieLength);
@@ -930,7 +997,7 @@ SSIZE_T rdstls_parse_pdu(wLog* log, wStream* stream)
 	wStream sbuffer = { 0 };
 	wStream* s = Stream_StaticConstInit(&sbuffer, Stream_Buffer(stream), Stream_Length(stream));
 
-	UINT16 version;
+	UINT16 version = 0;
 	if (Stream_GetRemainingLength(s) < 2)
 		return 0;
 	Stream_Read_UINT16(s, version);
@@ -940,7 +1007,7 @@ SSIZE_T rdstls_parse_pdu(wLog* log, wStream* stream)
 		return -1;
 	}
 
-	UINT16 pduType;
+	UINT16 pduType = 0;
 	if (Stream_GetRemainingLength(s) < 2)
 		return 0;
 	Stream_Read_UINT16(s, pduType);
@@ -952,7 +1019,7 @@ SSIZE_T rdstls_parse_pdu(wLog* log, wStream* stream)
 		case RDSTLS_TYPE_AUTHREQ:
 			if (Stream_GetRemainingLength(s) < 2)
 				return 0;
-			UINT16 dataType;
+			UINT16 dataType = 0;
 			Stream_Read_UINT16(s, dataType);
 			pduLength = rdstls_parse_pdu_data_type(log, dataType, s);
 
