@@ -19,6 +19,7 @@ static const size_t bool_list_indices[] = {
 	FreeRDP_AutoDenyCertificate,
 	FreeRDP_AutoLogonEnabled,
 	FreeRDP_AutoReconnectionEnabled,
+	FreeRDP_AutoReconnectionPacketSupported,
 	FreeRDP_BitmapCacheEnabled,
 	FreeRDP_BitmapCachePersistEnabled,
 	FreeRDP_BitmapCacheV3Enabled,
@@ -68,6 +69,7 @@ static const size_t bool_list_indices[] = {
 	FreeRDP_GatewayHttpExtAuthSspiNtlm,
 	FreeRDP_GatewayHttpTransport,
 	FreeRDP_GatewayHttpUseWebsockets,
+	FreeRDP_GatewayIgnoreRedirectionPolicy,
 	FreeRDP_GatewayRpcTransport,
 	FreeRDP_GatewayUdpTransport,
 	FreeRDP_GatewayUseSameCredentials,
@@ -79,6 +81,7 @@ static const size_t bool_list_indices[] = {
 	FreeRDP_GfxProgressiveV2,
 	FreeRDP_GfxSendQoeAck,
 	FreeRDP_GfxSmallCache,
+	FreeRDP_GfxSuspendFrameAck,
 	FreeRDP_GfxThinClient,
 	FreeRDP_GrabKeyboard,
 	FreeRDP_GrabMouse,
@@ -182,6 +185,7 @@ static const size_t bool_list_indices[] = {
 	FreeRDP_ToggleFullscreen,
 	FreeRDP_TransportDump,
 	FreeRDP_TransportDumpReplay,
+	FreeRDP_TransportDumpReplayNodelay,
 	FreeRDP_UnicodeInput,
 	FreeRDP_UnmapButtons,
 	FreeRDP_UseCommonStdioCallbacks,
@@ -253,6 +257,7 @@ static const size_t uint32_list_indices[] = {
 	FreeRDP_ExtEncryptionMethods,
 	FreeRDP_FakeMouseMotionInterval,
 	FreeRDP_Floatbar,
+	FreeRDP_ForceIPvX,
 	FreeRDP_FrameAcknowledge,
 	FreeRDP_GatewayAcceptedCertLength,
 	FreeRDP_GatewayCredentialsSource,
@@ -313,6 +318,7 @@ static const size_t uint32_list_indices[] = {
 	FreeRDP_RemoteFxCaptureFlags,
 	FreeRDP_RemoteFxCodecId,
 	FreeRDP_RemoteFxCodecMode,
+	FreeRDP_RemoteFxRlgrMode,
 	FreeRDP_RemoteWndSupportLevel,
 	FreeRDP_RequestedProtocols,
 	FreeRDP_SelectedProtocol,
@@ -326,6 +332,7 @@ static const size_t uint32_list_indices[] = {
 	FreeRDP_SmartSizingWidth,
 	FreeRDP_StaticChannelArraySize,
 	FreeRDP_StaticChannelCount,
+	FreeRDP_SurfaceCommandsSupported,
 	FreeRDP_TargetNetAddressCount,
 	FreeRDP_TcpAckTimeout,
 	FreeRDP_TcpConnectTimeout,
