@@ -44,6 +44,8 @@
 
 #define TAG FREERDP_TAG("core.update")
 
+#define FORCE_ASYNC_UPDATE_OFF
+
 static const char* const UPDATE_TYPE_STRINGS[] = { "Orders", "Bitmap", "Palette", "Synchronize" };
 
 static const char* update_type_to_string(UINT16 updateType)
@@ -947,8 +949,15 @@ BOOL update_post_connect(rdpUpdate* update)
 	up->asynchronous = update->context->settings->AsyncUpdate;
 
 	if (up->asynchronous)
+	{
+#if defined(FORCE_ASYNC_UPDATE_OFF)
+		WLog_WARN(TAG, "AsyncUpdate requested, but forced deactivated");
+		WLog_WARN(TAG, "see https://github.com/FreeRDP/FreeRDP/issues/10153 for details");
+#else
 		if (!(up->proxy = update_message_proxy_new(update)))
 			return FALSE;
+#endif
+	}
 
 	altsec->switch_surface.bitmapId = SCREEN_BITMAP_SURFACE;
 	IFCALL(update->altsec->SwitchSurface, update->context, &(altsec->switch_surface));
@@ -966,7 +975,11 @@ void update_post_disconnect(rdpUpdate* update)
 	up->asynchronous = update->context->settings->AsyncUpdate;
 
 	if (up->asynchronous)
+	{
+#if !defined(FORCE_ASYNC_UPDATE_OFF)
 		update_message_proxy_free(up->proxy);
+#endif
+	}
 
 	up->initialState = TRUE;
 }
@@ -2031,7 +2044,9 @@ static BOOL update_send_cache_brush(rdpContext* context, const CACHE_BRUSH_ORDER
 		return FALSE;
 
 	const size_t em = Stream_GetPosition(s);
-	WINPR_ASSERT(em > bm + 13);
+	if (em <= bm + 13)
+		return FALSE;
+
 	const size_t orderLength = (em - bm) - 13;
 	WINPR_ASSERT(orderLength <= UINT16_MAX);
 	Stream_SetPosition(s, bm);
@@ -3299,6 +3314,9 @@ void update_free(rdpUpdate* update)
 
 		MessageQueue_Free(up->queue);
 		DeleteCriticalSection(&up->mux);
+
+		if (up->us)
+			Stream_Free(up->us, TRUE);
 		free(update);
 	}
 }
@@ -3322,9 +3340,15 @@ BOOL update_begin_paint(rdpUpdate* update)
 
 	WINPR_ASSERT(update->context);
 
+	BOOL rc = IFCALLRESULT(TRUE, update->BeginPaint, update->context);
+	if (!rc)
+		WLog_WARN(TAG, "BeginPaint call failed");
+
 	/* Reset the invalid regions, we start a new frame here. */
 	rdpGdi* gdi = update->context->gdi;
-	WINPR_ASSERT(gdi);
+	if (!gdi)
+		return FALSE;
+
 	if (gdi->hdc && gdi->primary && gdi->primary->hdc)
 	{
 		HGDI_WND hwnd = gdi->primary->hdc->hwnd;
@@ -3335,10 +3359,6 @@ BOOL update_begin_paint(rdpUpdate* update)
 		hwnd->ninvalid = 0;
 	}
 
-	BOOL rc = IFCALLRESULT(TRUE, update->BeginPaint, update->context);
-	if (!rc)
-		WLog_WARN(TAG, "BeginPaint call failed");
-
 	return rc;
 }
 
