@@ -127,7 +127,8 @@ static BOOL test_peer_context_new(freerdp_peer* client, rdpContext* ctx)
 	                       SAMPLE_SERVER_DEFAULT_HEIGHT))
 		goto fail;
 
-	rfx_context_set_mode(context->rfx_context, RLGR3);
+	const UINT32 rlgr = freerdp_settings_get_uint32(ctx->settings, FreeRDP_RemoteFxRlgrMode);
+	rfx_context_set_mode(context->rfx_context, rlgr);
 
 	if (!(context->nsc_context = nsc_context_new()))
 		goto fail;
@@ -210,6 +211,13 @@ static void test_peer_end_frame(freerdp_peer* client)
 	context->frame_id++;
 }
 
+static BOOL stream_surface_bits_supported(const rdpSettings* settings)
+{
+	const UINT32 supported =
+	    freerdp_settings_get_uint32(settings, FreeRDP_SurfaceCommandsSupported);
+	return ((supported & SURFCMDS_STREAM_SURFACE_BITS) != 0);
+}
+
 static BOOL test_peer_draw_background(freerdp_peer* client)
 {
 	size_t size = 0;
@@ -256,7 +264,7 @@ static BOOL test_peer_draw_background(freerdp_peer* client)
 
 	memset(rgb_data, 0xA0, size);
 
-	if (RemoteFxCodec)
+	if (RemoteFxCodec && stream_surface_bits_supported(settings))
 	{
 		WLog_DBG(TAG, "Using RemoteFX codec");
 		rfx_context_set_pixel_format(context->rfx_context, colorFormat);
@@ -407,8 +415,9 @@ static void test_peer_draw_icon(freerdp_peer* client, UINT32 x, UINT32 y)
 		return;
 
 	test_peer_begin_frame(client);
+
 	const BOOL RemoteFxCodec = freerdp_settings_get_bool(settings, FreeRDP_RemoteFxCodec);
-	if (RemoteFxCodec)
+	if (RemoteFxCodec && stream_surface_bits_supported(settings))
 	{
 		const UINT32 RemoteFxCodecId =
 		    freerdp_settings_get_uint32(settings, FreeRDP_RemoteFxCodecId);
@@ -537,6 +546,7 @@ static BOOL test_sleep_tsdiff(UINT32* old_sec, UINT32* old_usec, UINT32 new_sec,
 
 static BOOL tf_peer_dump_rfx(freerdp_peer* client)
 {
+	BOOL rc = FALSE;
 	wStream* s = NULL;
 	UINT32 prev_seconds = 0;
 	UINT32 prev_useconds = 0;
@@ -559,8 +569,9 @@ static BOOL tf_peer_dump_rfx(freerdp_peer* client)
 	update = client->context->update;
 	WINPR_ASSERT(update);
 
-	if (!(pcap_rfx = pcap_open(info->test_pcap_file, FALSE)))
-		return FALSE;
+	pcap_rfx = pcap_open(info->test_pcap_file, FALSE);
+	if (!pcap_rfx)
+		goto fail;
 
 	prev_seconds = prev_useconds = 0;
 
@@ -589,9 +600,11 @@ static BOOL tf_peer_dump_rfx(freerdp_peer* client)
 			break;
 	}
 
+	rc = TRUE;
+fail:
 	Stream_Free(s, TRUE);
 	pcap_close(pcap_rfx);
-	return TRUE;
+	return rc;
 }
 
 static DWORD WINAPI tf_debug_channel_thread_func(LPVOID arg)
