@@ -55,6 +55,14 @@ static int shadow_server_print_command_line_help(int argc, char** argv,
 
 	printf("Usage: %s [options]\n", argv[0]);
 	printf("\n");
+	printf("Notes: By default NLA security is active.\n");
+	printf("\tIn this mode a SAM database is required.\n");
+	printf("\tProvide one with /sam-file:<file with path>\n");
+	printf("\telse the default path /etc/winpr/SAM is used.\n");
+	printf("\tIf there is no existing SAM file authentication for all users will fail.\n");
+	printf(
+	    "\n\tIf authentication against PAM is desired, start with -sec-nla (requires compiled in "
+	    "support for PAM)\n\n");
 	printf("Syntax:\n");
 	printf("    /flag (enables flag)\n");
 	printf("    /option:<value> (specifies option with value)\n");
@@ -404,6 +412,23 @@ int shadow_server_parse_command_line(rdpShadowServer* server, int argc, char** a
 			if (!WLog_AddStringLogFilters(arg->Value))
 				return COMMAND_LINE_ERROR;
 		}
+		CommandLineSwitchCase(arg, "nsc")
+		{
+			if (!freerdp_settings_set_bool(settings, FreeRDP_NSCodec, arg->Value ? TRUE : FALSE))
+				return COMMAND_LINE_ERROR;
+		}
+		CommandLineSwitchCase(arg, "rfx")
+		{
+			if (!freerdp_settings_set_bool(settings, FreeRDP_RemoteFxCodec,
+			                               arg->Value ? TRUE : FALSE))
+				return COMMAND_LINE_ERROR;
+		}
+		CommandLineSwitchCase(arg, "gfx")
+		{
+			if (!freerdp_settings_set_bool(settings, FreeRDP_SupportGraphicsPipeline,
+			                               arg->Value ? TRUE : FALSE))
+				return COMMAND_LINE_ERROR;
+		}
 		CommandLineSwitchCase(arg, "gfx-progressive")
 		{
 			if (!freerdp_settings_set_bool(settings, FreeRDP_GfxProgressive,
@@ -726,58 +751,9 @@ int shadow_server_stop(rdpShadowServer* server)
 
 static int shadow_server_init_config_path(rdpShadowServer* server)
 {
-#ifdef _WIN32
-
 	if (!server->ConfigPath)
 	{
-		server->ConfigPath = GetEnvironmentSubPath("LOCALAPPDATA", "freerdp");
-	}
-
-#endif
-#ifdef __APPLE__
-
-	if (!server->ConfigPath)
-	{
-		char* userLibraryPath;
-		char* userApplicationSupportPath;
-		userLibraryPath = GetKnownSubPath(KNOWN_PATH_HOME, "Library");
-
-		if (userLibraryPath)
-		{
-			if (!winpr_PathFileExists(userLibraryPath) && !winpr_PathMakePath(userLibraryPath, 0))
-			{
-				WLog_ERR(TAG, "Failed to create directory '%s'", userLibraryPath);
-				free(userLibraryPath);
-				return -1;
-			}
-
-			userApplicationSupportPath = GetCombinedPath(userLibraryPath, "Application Support");
-
-			if (userApplicationSupportPath)
-			{
-				if (!winpr_PathFileExists(userApplicationSupportPath) &&
-				    !winpr_PathMakePath(userApplicationSupportPath, 0))
-				{
-					WLog_ERR(TAG, "Failed to create directory '%s'", userApplicationSupportPath);
-					free(userLibraryPath);
-					free(userApplicationSupportPath);
-					return -1;
-				}
-
-				server->ConfigPath = GetCombinedPath(userApplicationSupportPath, "freerdp");
-			}
-
-			free(userLibraryPath);
-			free(userApplicationSupportPath);
-		}
-	}
-
-#endif
-
-	if (!server->ConfigPath)
-	{
-		char* configHome = NULL;
-		configHome = GetKnownPath(KNOWN_PATH_XDG_CONFIG_HOME);
+		char* configHome = freerdp_settings_get_config_path();
 
 		if (configHome)
 		{
@@ -788,8 +764,7 @@ static int shadow_server_init_config_path(rdpShadowServer* server)
 				return -1;
 			}
 
-			server->ConfigPath = GetKnownSubPath(KNOWN_PATH_XDG_CONFIG_HOME, "freerdp");
-			free(configHome);
+			server->ConfigPath = configHome;
 		}
 	}
 
@@ -1005,7 +980,6 @@ rdpShadowServer* shadow_server_new(void)
 	server->port = 3389;
 	server->mayView = TRUE;
 	server->mayInteract = TRUE;
-	server->rfxMode = RLGR3;
 	server->h264RateControlMode = H264_RATECONTROL_VBR;
 	server->h264BitRate = 10000000;
 	server->h264FrameRate = 30;
