@@ -39,6 +39,7 @@
 #include <winpr/crypto.h>
 #include <winpr/path.h>
 #include <winpr/wtypes.h>
+#include <winpr/winsock.h>
 
 #include "kerberos.h"
 
@@ -196,6 +197,25 @@ static INLINE krb5glue_key get_key(struct krb5glue_keyset* keyset)
 	                               : keyset->session_key;
 }
 
+static BOOL isValidIPv4(const char* ipAddress)
+{
+	struct sockaddr_in sa = { 0 };
+	int result = inet_pton(AF_INET, ipAddress, &(sa.sin_addr));
+	return result != 0;
+}
+
+static BOOL isValidIPv6(const char* ipAddress)
+{
+	struct sockaddr_in6 sa = { 0 };
+	int result = inet_pton(AF_INET6, ipAddress, &(sa.sin6_addr));
+	return result != 0;
+}
+
+static BOOL isValidIP(const char* ipAddress)
+{
+	return isValidIPv4(ipAddress) || isValidIPv6(ipAddress);
+}
+
 #endif /* WITH_KRB5 */
 
 static SECURITY_STATUS SEC_ENTRY kerberos_AcquireCredentialsHandleA(
@@ -394,7 +414,7 @@ static SECURITY_STATUS SEC_ENTRY kerberos_AcquireCredentialsHandleW(
     void* pAuthData, SEC_GET_KEY_FN pGetKeyFn, void* pvGetKeyArgument, PCredHandle phCredential,
     PTimeStamp ptsExpiry)
 {
-	SECURITY_STATUS status = 0;
+	SECURITY_STATUS status = SEC_E_INSUFFICIENT_MEMORY;
 	char* principal = NULL;
 	char* package = NULL;
 
@@ -402,23 +422,22 @@ static SECURITY_STATUS SEC_ENTRY kerberos_AcquireCredentialsHandleW(
 	{
 		principal = ConvertWCharToUtf8Alloc(pszPrincipal, NULL);
 		if (!principal)
-			return SEC_E_INSUFFICIENT_MEMORY;
+			goto fail;
 	}
 	if (pszPackage)
 	{
 		package = ConvertWCharToUtf8Alloc(pszPackage, NULL);
 		if (!package)
-			return SEC_E_INSUFFICIENT_MEMORY;
+			goto fail;
 	}
 
 	status =
 	    kerberos_AcquireCredentialsHandleA(principal, package, fCredentialUse, pvLogonID, pAuthData,
 	                                       pGetKeyFn, pvGetKeyArgument, phCredential, ptsExpiry);
 
-	if (principal)
-		free(principal);
-	if (package)
-		free(package);
+fail:
+	free(principal);
+	free(package);
 
 	return status;
 }
@@ -657,6 +676,7 @@ static BOOL kerberos_rd_tgt_token(const sspi_gss_data* token, char** target, krb
 
 		*buf++ = '@';
 		strcpy(buf, str);
+		free(str);
 		return TRUE;
 	}
 	else if (val == KRB_TGT_REP)
@@ -821,6 +841,11 @@ static SECURITY_STATUS SEC_ENTRY kerberos_InitializeSecurityContextA(
 		}
 		else
 			host = target;
+		if (isValidIP(host))
+		{
+			status = SEC_E_NO_CREDENTIALS;
+			goto cleanup;
+		}
 	}
 
 	/* SSPI flags are compatible with GSS flags except INTEG_FLAG */
@@ -1303,6 +1328,9 @@ static KRB_CONTEXT* get_context(PCtxtHandle phContext)
 		return NULL;
 
 	TCHAR* name = sspi_SecureHandleGetUpperPointer(phContext);
+	if (!name)
+		return NULL;
+
 	if (_tcscmp(KERBEROS_SSP_NAME, name) != 0)
 		return NULL;
 	return sspi_SecureHandleGetLowerPointer(phContext);
@@ -1492,7 +1520,7 @@ static SECURITY_STATUS SEC_ENTRY kerberos_EncryptMessage(PCtxtHandle phContext,
 	KRB_CONTEXT* context = get_context(phContext);
 	PSecBuffer sig_buffer = NULL;
 	PSecBuffer data_buffer = NULL;
-	BYTE* header = NULL;
+	char* header = NULL;
 	BYTE flags = 0;
 	krb5glue_key key = NULL;
 	krb5_keyusage usage = 0;
@@ -1543,15 +1571,15 @@ static SECURITY_STATUS SEC_ENTRY kerberos_EncryptMessage(PCtxtHandle phContext,
 	/* Set up the iov array in sig_buffer */
 	header = sig_buffer->pvBuffer;
 	encrypt_iov[2].data.data = header + 16;
-	encrypt_iov[3].data.data = (BYTE*)encrypt_iov[2].data.data + encrypt_iov[2].data.length;
-	encrypt_iov[4].data.data = (BYTE*)encrypt_iov[3].data.data + encrypt_iov[3].data.length;
-	encrypt_iov[0].data.data = (BYTE*)encrypt_iov[4].data.data + encrypt_iov[4].data.length;
+	encrypt_iov[3].data.data = encrypt_iov[2].data.data + encrypt_iov[2].data.length;
+	encrypt_iov[4].data.data = encrypt_iov[3].data.data + encrypt_iov[3].data.length;
+	encrypt_iov[0].data.data = encrypt_iov[4].data.data + encrypt_iov[4].data.length;
 	encrypt_iov[1].data.data = data_buffer->pvBuffer;
 
 	/* Write the GSS header with 0 in RRC */
 	Data_Write_UINT16_BE(header, TOK_ID_WRAP);
 	header[2] = flags;
-	header[3] = 0xFF;
+	header[3] = (char)0xFF;
 	Data_Write_UINT32(header + 4, 0);
 	Data_Write_UINT64_BE(header + 8, (context->local_seq + MessageSeqNo));
 
@@ -1652,8 +1680,8 @@ static SECURITY_STATUS SEC_ENTRY kerberos_DecryptMessage(PCtxtHandle phContext,
 	iov[0].data.data = header + 16 + rrc + ec;
 	iov[1].data.data = data_buffer->pvBuffer;
 	iov[2].data.data = header + 16 + ec;
-	iov[3].data.data = (BYTE*)iov[2].data.data + iov[2].data.length;
-	iov[4].data.data = (BYTE*)iov[3].data.data + iov[3].data.length;
+	iov[3].data.data = iov[2].data.data + iov[2].data.length;
+	iov[4].data.data = iov[3].data.data + iov[3].data.length;
 
 	if (krb_log_exec(krb5glue_decrypt_iov, context->ctx, key, usage, iov, ARRAYSIZE(iov)))
 		return SEC_E_INTERNAL_ERROR;
